UPDATE table_version SET version_number = '54' WHERE version_id = '1';

CREATE TABLE table_expl (
  expl_id BIGINT NOT NULL IDENTITY,
  expl_name VARCHAR(128) NOT NULL,
  folder_id BIGINT NOT NULL,
  sut_id BIGINT NOT NULL,
  tester_id BIGINT NOT NULL
);
ALTER TABLE table_expl ADD CONSTRAINT PRIMARY_89 PRIMARY KEY (expl_id);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl TO PUBLIC;

CREATE TABLE table_expl_data (
  data_id BIGINT NOT NULL IDENTITY,
  expl_id BIGINT NOT NULL,
  status TINYINT DEFAULT '10' NOT NULL,
  priority TINYINT DEFAULT '20' NOT NULL,
  charter TEXT,
  notes TEXT,
  percent_setup TINYINT DEFAULT '100' NOT NULL,
  percent_exec TINYINT DEFAULT '0' NOT NULL,
  percent_report TINYINT DEFAULT '0' NOT NULL,
  percent_other TINYINT DEFAULT '0' NOT NULL,
  startDate DATETIME,
  stopDate DATETIME,
  cummul_runtime BIGINT DEFAULT '0' NOT NULL,
  date_submitted DATETIME,
  revision BIGINT DEFAULT '1' NOT NULL
);
ALTER TABLE table_expl_data ADD CONSTRAINT PRIMARY_90 PRIMARY KEY (data_id);
CREATE UNIQUE INDEX expl_data_UK_1 ON table_expl_data (expl_id, revision);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl_data TO PUBLIC;

CREATE TABLE table_def_int_expl (
  defect_id BIGINT NOT NULL,
  expl_id BIGINT NOT NULL
);
CREATE INDEX def_int_expl_IK_1 ON table_def_int_expl(defect_id);
CREATE INDEX def_int_expl_IK_2 ON table_def_int_expl(expl_id);
CREATE UNIQUE INDEX def_int_expl_UK_1 ON table_def_int_expl (defect_id, expl_id);

CREATE TABLE table_expl_attach (
  attachment_id BIGINT NOT NULL,
  expl_id BIGINT NOT NULL,
  attachment_name VARCHAR(128) NOT NULL,
  locked_by BIGINT,
  hidden TINYINT DEFAULT '0' NOT NULL
);
ALTER TABLE table_expl_attach ADD CONSTRAINT PRIMARY_91 PRIMARY KEY (attachment_id);
CREATE INDEX expl_attach_IK_1 ON table_expl_attach (locked_by);
CREATE INDEX expl_attach_IK_2 ON table_expl_attach (expl_id);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl_attach TO PUBLIC;

CREATE TABLE table_expl_attach_data (
  data_id BIGINT NOT NULL,
  attachment_id BIGINT NOT NULL,
  attachment_data VARBINARY(MAX),
  date_submitted DATETIME,
  size_data BIGINT NOT NULL,
  author_id BIGINT NOT NULL,
  revision BIGINT DEFAULT '1' NOT NULL
);
ALTER TABLE table_expl_attach_data ADD CONSTRAINT PRIMARY_92 PRIMARY KEY (data_id);
CREATE INDEX expl_attach_data_IK_1 ON table_expl_attach_data (attachment_id);
CREATE INDEX expl_attach_data_IK_2 ON table_expl_attach_data (author_id);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl_attach_data TO PUBLIC;


CREATE TABLE table_expl_attr_boolean (
  expl_id BIGINT NOT NULL,
  attr_boolean_id BIGINT NOT NULL,
  expl_attr_boolean_value TINYINT DEFAULT '0' NOT NULL
);
CREATE INDEX expl_attr_boolean_IK_1 ON table_expl_attr_boolean (attr_boolean_id);
CREATE INDEX expl_attr_boolean_IK_2 ON table_expl_attr_boolean (expl_id);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl_attr_boolean TO PUBLIC;

CREATE TABLE table_expl_attr_integer (
  expl_id BIGINT NOT NULL,
  attr_integer_id BIGINT NOT NULL,
  expl_attr_integer_value TEXT NOT NULL
);
CREATE INDEX expl_attr_integer_IK_1 ON table_expl_attr_integer (attr_integer_id);
CREATE INDEX expl_attr_integer_IK_2 ON table_expl_attr_integer (expl_id);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl_attr_integer TO PUBLIC;

CREATE TABLE table_expl_attr_string (
  expl_id BIGINT NOT NULL,
  attr_string_id BIGINT NOT NULL,
  expl_attr_string_value TEXT
);
CREATE INDEX expl_attr_string_IK_1 ON table_expl_attr_string (attr_string_id);
CREATE INDEX expl_attr_string_IK_2 ON table_expl_attr_string (expl_id);
GRANT SELECT, INSERT, DELETE, UPDATE, REFERENCES ON table_expl_attr_string TO PUBLIC;


ALTER TABLE table_expl ADD CONSTRAINT table_expl_ibfk_1 FOREIGN KEY (folder_id) REFERENCES table_folder(folder_id); -- TRIGGER REPLACING ON DELETE;
ALTER TABLE table_expl ADD CONSTRAINT table_expl_ibfk_2 FOREIGN KEY (sut_id)    REFERENCES table_sut(sut_id); -- TRIGGER REPLACING ON DELETE;
ALTER TABLE table_expl ADD CONSTRAINT table_expl_ibfk_3 FOREIGN KEY (tester_id) REFERENCES table_user(user_id); -- TRIGGER REPLACING ON DELETE;

ALTER TABLE table_expl_data ADD CONSTRAINT table_expl_data_ibfk_1 FOREIGN KEY (expl_id) REFERENCES table_expl(expl_id) ON DELETE CASCADE;

ALTER TABLE table_def_int_expl ADD CONSTRAINT table_def_int_expl_ibfk_1 FOREIGN KEY (defect_id) REFERENCES table_def_int(defect_id) ON DELETE CASCADE;
ALTER TABLE table_def_int_expl ADD CONSTRAINT table_def_int_expl_ibfk_2 FOREIGN KEY (expl_id)   REFERENCES table_expl(expl_id); -- TRIGGER REPLACING ON DELETE;

ALTER TABLE table_expl_attach ADD CONSTRAINT table_expl_attach_ibfk_1 FOREIGN KEY (locked_by) REFERENCES table_user (user_id) ON DELETE CASCADE;
ALTER TABLE table_expl_attach ADD CONSTRAINT table_expl_attach_ibfk_2 FOREIGN KEY (expl_id) REFERENCES table_expl (expl_id); -- TRIGGER REPLACING ON DELETE;

ALTER TABLE table_expl_attach_data ADD CONSTRAINT table_expl_attach_data_ibfk_1 FOREIGN KEY (attachment_id) REFERENCES table_expl_attach (attachment_id) ON DELETE CASCADE;
ALTER TABLE table_expl_attach_data ADD CONSTRAINT table_expl_attach_data_ibfk_2 FOREIGN KEY (author_id) REFERENCES table_user (user_id); -- TRIGGER REPLACING ON DELETE;

ALTER TABLE table_expl_attr_boolean ADD CONSTRAINT table_expl_attr_boolean_ibfk_1 FOREIGN KEY (attr_boolean_id) REFERENCES table_attr_boolean (attr_boolean_id) ON DELETE CASCADE;
ALTER TABLE table_expl_attr_boolean ADD CONSTRAINT table_expl_attr_boolean_ibfk_2 FOREIGN KEY (expl_id) REFERENCES table_expl (expl_id); -- TRIGGER REPLACING ON DELETE;

ALTER TABLE table_expl_attr_integer ADD CONSTRAINT table_expl_attr_integer_ibfk_1 FOREIGN KEY (attr_integer_id) REFERENCES table_attr_integer (attr_integer_id) ON DELETE CASCADE;
ALTER TABLE table_expl_attr_integer ADD CONSTRAINT table_expl_attr_integer_ibfk_2 FOREIGN KEY (expl_id) REFERENCES table_expl (expl_id); -- TRIGGER REPLACING ON DELETE;

ALTER TABLE table_expl_attr_string ADD CONSTRAINT table_expl_attr_string_ibfk_1 FOREIGN KEY (attr_string_id) REFERENCES table_attr_string (attr_string_id) ON DELETE CASCADE;
ALTER TABLE table_expl_attr_string ADD CONSTRAINT table_expl_attr_string_ibfk_2 FOREIGN KEY (expl_id) REFERENCES table_expl (expl_id); -- TRIGGER REPLACING ON DELETE;

DROP TRIGGER trig_del_sut;
CREATE TRIGGER trig_del_sut               ON table_sut INSTEAD OF DELETE AS
   DELETE FROM table_sch                  WHERE EXISTS (SELECT * FROM DELETED WHERE table_sch.sut_id                             = DELETED.sut_id)
   DELETE FROM table_sess                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_sess.sut_id                            = DELETED.sut_id)
   DELETE FROM table_def_int_sut          WHERE EXISTS (SELECT * FROM DELETED WHERE table_def_int_sut.sut_id                     = DELETED.sut_id)
   DELETE FROM table_expl                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl.sut_id                            = DELETED.sut_id)
   DELETE FROM table_sut                  WHERE EXISTS (SELECT * FROM DELETED WHERE table_sut.sut_id                             = DELETED.sut_id);

DROP TRIGGER trig_del_user;
CREATE TRIGGER trig_del_user              ON table_user INSTEAD OF DELETE AS
   DELETE FROM table_sch                  WHERE EXISTS (SELECT * FROM DELETED WHERE table_sch.user_id                            = DELETED.user_id)
   DELETE FROM table_sch_ccemails         WHERE EXISTS (SELECT * FROM DELETED WHERE table_sch_ccemails.user_id                   = DELETED.user_id)
   DELETE FROM table_sess                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_sess.user_id                           = DELETED.user_id)
   DELETE FROM table_sess_ccemails        WHERE EXISTS (SELECT * FROM DELETED WHERE table_sess_ccemails.user_id                  = DELETED.user_id)
   DELETE FROM table_sprint_task          WHERE EXISTS (SELECT * FROM DELETED WHERE table_sprint_task.assigned_to                = DELETED.user_id)
   DELETE FROM table_test                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_test.author_id                         = DELETED.user_id)
   DELETE FROM table_test                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_test.developer_id                      = DELETED.user_id)
   DELETE FROM table_def_int_assigned_to  WHERE EXISTS (SELECT * FROM DELETED WHERE table_def_int_assigned_to.assigned_to        = DELETED.user_id)
   DELETE FROM table_def_int              WHERE EXISTS (SELECT * FROM DELETED WHERE table_def_int.reporter                       = DELETED.user_id)
   DELETE FROM table_expl                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl.tester_id                         = DELETED.user_id)
   DELETE FROM table_user                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_user.user_id                           = DELETED.user_id);

CREATE TRIGGER trig_del_expl              ON table_expl INSTEAD OF DELETE AS
   DELETE FROM table_def_int_expl         WHERE EXISTS (SELECT * FROM DELETED WHERE table_def_int_expl.expl_id                   = DELETED.expl_id)
   DELETE FROM table_expl_attach          WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl_attach.expl_id                    = DELETED.expl_id)
   DELETE FROM table_expl_attr_boolean    WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl_attr_boolean.expl_id              = DELETED.expl_id)
   DELETE FROM table_expl_attr_integer    WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl_attr_integer.expl_id              = DELETED.expl_id)
   DELETE FROM table_expl_attr_string     WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl_attr_string.expl_id               = DELETED.expl_id)
   DELETE FROM table_expl                 WHERE EXISTS (SELECT * FROM DELETED WHERE table_expl.expl_id                           = DELETED.expl_id);

