UPDATE table_version SET version_number = '15' WHERE version_id = '1';

CREATE TABLE table_project (
  project_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  folder_id INTEGER UNSIGNED NOT NULL,
  project_name VARCHAR(128) NOT NULL,
  project_description TEXT NULL,
  focus_ratio TINYINT UNSIGNED NOT NULL DEFAULT 80,
  PRIMARY KEY(project_id),
  INDEX project_FKIndex2(folder_id),
  FOREIGN KEY(folder_id)
    REFERENCES table_folder(folder_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=InnoDB;

ALTER TABLE table_folder ADD project_id INTEGER UNSIGNED NOT NULL DEFAULT 1;
ALTER TABLE table_folder ADD INDEX folder_FKIndex4(project_id);

INSERT INTO table_project VALUES (1, 1, 'na', 'na', 80);

ALTER TABLE table_folder ADD FOREIGN KEY(project_id) REFERENCES table_project(project_id) ON DELETE CASCADE ON UPDATE NO ACTION;

INSERT INTO table_folder VALUES (9, 9, 'na', 1, 1, 1); #project
INSERT INTO table_folder VALUES (10, 10, 'na', 1, 1, 1); #task

CREATE TABLE table_task (
  task_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  folder_id INTEGER UNSIGNED NOT NULL,
  task_name VARCHAR(128) NOT NULL,
  task_description TEXT NULL,
  priority TINYINT NOT NULL,
  estimated DECIMAL(3,1) NOT NULL DEFAULT 1.0,
  PRIMARY KEY(task_id),
  INDEX task_FKIndex1(folder_id),
  FOREIGN KEY(folder_id)
    REFERENCES table_folder(folder_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=InnoDB;



CREATE TABLE table_sprint (
  sprint_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  sprint_name VARCHAR(128) NOT NULL,
  sprint_description TEXT NULL,
  status TINYINT NOT NULL,
  start_date DATETIME NULL,
  stop_date DATETIME NULL,
  project_id INTEGER UNSIGNED NOT NULL,
  PRIMARY KEY(sprint_id),
  INDEX table_sprint_FKIndex1(project_id),
  FOREIGN KEY(project_id)
    REFERENCES table_project(project_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=InnoDB;

CREATE TABLE table_sprint_task (
  sprint_id INTEGER UNSIGNED NOT NULL,
  task_id INTEGER UNSIGNED NOT NULL,
  progress TINYINT UNSIGNED NOT NULL DEFAULT 0,
  INDEX sprint_task_FKIndex1(sprint_id),
  INDEX sprint_task_FKIndex2(task_id),
  FOREIGN KEY(sprint_id)
    REFERENCES table_sprint(sprint_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY(task_id)
    REFERENCES table_task(task_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=InnoDB;

CREATE TABLE table_sprint_developer (
  sprint_id INTEGER UNSIGNED NOT NULL,
  user_id INTEGER UNSIGNED NOT NULL,
  availability TINYINT UNSIGNED NOT NULL DEFAULT 0,
  INDEX sprint_user_FKIndex1(sprint_id),
  INDEX sprint_user_FKIndex2(user_id),
  FOREIGN KEY(sprint_id)
    REFERENCES table_sprint(sprint_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY(user_id)
    REFERENCES table_user(user_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=InnoDB;

CREATE TABLE table_sprint_stat (
  sprint_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  date DATETIME NULL,
  stat MEDIUMTEXT NOT NULL,
  FOREIGN KEY(sprint_id)
    REFERENCES table_sprint(sprint_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=InnoDB;