UPDATE table_version SET version_number = '54' WHERE version_id = '1';

CREATE TABLE table_expl (
  expl_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  expl_name VARCHAR(128) CHARACTER SET utf8 NOT NULL,
  folder_id INTEGER UNSIGNED NOT NULL,
  sut_id INTEGER UNSIGNED NOT NULL,
  tester_id INTEGER UNSIGNED NOT NULL,
  PRIMARY KEY(expl_id),
  INDEX expl_IK_1(folder_id),
  INDEX expl_IK_2(sut_id),
  INDEX expl_IK_3(tester_id),
  FOREIGN KEY (folder_id)
    REFERENCES table_folder(folder_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (sut_id)
    REFERENCES table_sut(sut_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (tester_id)
    REFERENCES table_user(user_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_expl_data (
  data_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  expl_id INTEGER UNSIGNED NOT NULL,
  status TINYINT NOT NULL DEFAULT 10,
  priority TINYINT NOT NULL DEFAULT 20,
  charter LONGTEXT CHARACTER SET utf8,
  notes LONGTEXT CHARACTER SET utf8,
  percent_setup TINYINT NOT NULL DEFAULT 100,
  percent_exec TINYINT NOT NULL DEFAULT 0,
  percent_report TINYINT NOT NULL DEFAULT 0,
  percent_other TINYINT NOT NULL DEFAULT 0,
  startDate DATETIME NULL,
  stopDate DATETIME NULL,
  cummul_runtime BIGINT UNSIGNED NOT NULL DEFAULT 0,
  date_submitted DATETIME NULL,
  revision INTEGER UNSIGNED NOT NULL DEFAULT 1,
  PRIMARY KEY(data_id),
  UNIQUE KEY expl_data_UK_1(expl_id, revision),
  INDEX expl_data_IK_1(expl_id),
  FOREIGN KEY (expl_id)
    REFERENCES table_expl(expl_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_def_int_expl (
  defect_id INTEGER UNSIGNED NOT NULL,
  expl_id INTEGER UNSIGNED NOT NULL,
  UNIQUE KEY def_int_expl_UK_1(defect_id, expl_id),
  INDEX def_int_expl_IK_1(defect_id),
  INDEX def_int_expl_IK_2(expl_id),
  FOREIGN KEY (defect_id)
    REFERENCES table_def_int(defect_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (expl_id)
    REFERENCES table_expl(expl_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_expl_attach (
  attachment_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  expl_id INTEGER UNSIGNED NOT NULL,
  attachment_name VARCHAR(128) CHARACTER SET utf8 NOT NULL,
  locked_by INTEGER UNSIGNED DEFAULT NULL,
  hidden BOOL NOT NULL DEFAULT 0,
  PRIMARY KEY(attachment_id),
  INDEX expl_attach_IK_1(locked_by),
  INDEX expl_attach_IK_2(expl_id),
  FOREIGN KEY (locked_by)
    REFERENCES table_user(user_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (expl_id)
    REFERENCES table_expl(expl_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_expl_attach_data (
  data_id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  attachment_id INTEGER UNSIGNED NOT NULL,
  attachment_data LONGBLOB,
  date_submitted DATETIME NULL,
  size_data INTEGER UNSIGNED NOT NULL,
  author_id INTEGER UNSIGNED NOT NULL,
  revision INTEGER UNSIGNED NOT NULL DEFAULT 1,
  PRIMARY KEY(data_id),
  INDEX expl_attach_data_IK_1(attachment_id),
  INDEX expl_attach_data_IK_2(author_id),
  FOREIGN KEY (attachment_id)
    REFERENCES table_expl_attach(attachment_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (author_id)
    REFERENCES table_user(user_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_expl_attr_boolean (
  expl_id INTEGER UNSIGNED NOT NULL,
  attr_boolean_id INTEGER UNSIGNED NOT NULL,
  expl_attr_boolean_value BOOL NOT NULL DEFAULT 0,
  INDEX expl_attr_boolean_IK_1(attr_boolean_id),
  INDEX expl_attr_boolean_IK_2(expl_id),
  FOREIGN KEY (attr_boolean_id)
    REFERENCES table_attr_boolean(attr_boolean_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (expl_id)
    REFERENCES table_expl(expl_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_expl_attr_integer (
  expl_id INTEGER UNSIGNED NOT NULL,
  attr_integer_id INTEGER UNSIGNED NOT NULL,
  expl_attr_integer_value TEXT CHARACTER SET utf8 NOT NULL,
  INDEX expl_attr_integer_IK_1(attr_integer_id),
  INDEX expl_attr_integer_IK_2(expl_id),
  FOREIGN KEY (expl_id)
    REFERENCES table_expl(expl_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (attr_integer_id)
    REFERENCES table_attr_integer(attr_integer_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;

CREATE TABLE table_expl_attr_string (
  expl_id INTEGER UNSIGNED NOT NULL,
  attr_string_id INTEGER UNSIGNED NOT NULL,
  expl_attr_string_value TEXT CHARACTER SET utf8,
  INDEX expl_attr_string_IK_1(attr_string_id),
  INDEX expl_attr_string_IK_2(expl_id),
  FOREIGN KEY (attr_string_id)
    REFERENCES table_attr_string(attr_string_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION,
  FOREIGN KEY (expl_id)
    REFERENCES table_expl(expl_id)
      ON DELETE CASCADE
      ON UPDATE NO ACTION
)
ENGINE=INNODB;


