<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="xml" indent="yes" encoding="UTF-8"/>

<!-- ============================ PRESENTATION ============================= -->

  <xsl:template name="big_hr">
    <br/>
    <table width="100%"><tr><td height="4px" class="table_data_header"/></tr></table>
    <br/>
  </xsl:template>

  <xsl:template name="small_hr">
    <table width="100%"><tr><td height="2px" class="table_data_subheader"/></tr></table>
  </xsl:template>

  <xsl:template name="very_small_hr">
    <table width="100%"><tr><td height="1px" class="table_data_subheader"/></tr></table>
  </xsl:template>

  <xsl:template name="percentage">
      <xsl:param name="max"/>
      <xsl:param name="value"/>

      <xsl:variable name="percent" select="format-number(100 * $value div $max, '##.##')" />
      <xsl:variable name="rest"    select="100 - $percent" />

      <table border="0" cellpading="0" cellspacing="0" width="160px">
         <tr>
            <!-- label -->
            <td height="14px" class="table_label" width="50px"> 
               <xsl:value-of select="$percent" /> %
            </td>

            <!-- progress bar -->
            <td width="100px">
              <table border="0" cellpading="0" cellspacing="0" width="100%">
                <tr>
                  <td height="14px" class="bg_percentage" >
                     <xsl:attribute name="width">
                        <xsl:value-of select="$percent" />%
                     </xsl:attribute>
                  </td>
                  <td height="14px" class="bg_percentage_empty">
                     <xsl:attribute name="width">
                        <xsl:value-of select="$rest" />%
                     </xsl:attribute>
                  </td>
                </tr>
              </table>
            </td>
         </tr>
      </table>
  </xsl:template>


  <!-- 
  generate something like: 
  <a href="http://192.168.10.1:8080/XSTUDIO/bin/xstudio.php?id=T_1234">[T_1234]</a> My_test
  -->
  <xsl:template name="urlToObject">
      <xsl:param name="globalId"/>
      <xsl:param name="name"/>
      
      <xsl:variable name="codebase" select="$codebase" /> <!-- variable inherited from the original template -->

      <xsl:if test="$codebase!='null'">
        <a class="no_style">
          <xsl:attribute name="href">
            <xsl:value-of select="concat($codebase, 'bin/xstudio.php?id=', $globalId)" />
          </xsl:attribute>
          <img border="0" align="top" src="resources/images/IconRedirect.gif"/>
        </a>
      </xsl:if>
      [<xsl:value-of select="$globalId"/>]&#160;<xsl:value-of select="$name"/>
  </xsl:template>

<!-- =========================== ID GENERATION ============================= -->

  <xsl:template name="generateUniqueId">
      <xsl:param name="type"/>
      <xsl:param name="id"/>

      <xsl:value-of select="concat($type, $id)" />
  </xsl:template>

  <xsl:template name="generateSharpUniqueId">
      <xsl:param name="type"/>
      <xsl:param name="id"/>

      <xsl:value-of select="concat('#', $type, $id)" />
  </xsl:template>


  <xsl:template name="generateUniqueIdFromTwoIds">
      <xsl:param name="type1"/>
      <xsl:param name="id1"/>
      <xsl:param name="type2"/>
      <xsl:param name="id2"/>

      <xsl:value-of select="concat($type1, $id1, '_', $type2, $id2)" />
  </xsl:template>

  <xsl:template name="generateSharpUniqueIdFromTwoIds">
      <xsl:param name="type1"/>
      <xsl:param name="id1"/>
      <xsl:param name="type2"/>
      <xsl:param name="id2"/>

      <xsl:value-of select="concat('#', $type1, $id1, '_', $type2, $id2)" />
  </xsl:template>

<!-- ============================ ATTACHMENTS ============================== -->

    <xsl:template name="generateAttachments">
      <xsl:param name="parentType"/>
      <xsl:param name="attachmentIcon"/>

      <xsl:variable name="nbAttachments" select="count(attachment[@hidden='0'])" />

      <xsl:if test="$nbAttachments > 0">
        <xsl:for-each select="attachment[@hidden='0']">
          <xsl:variable name="attachmentId"       select="@id" />
          <xsl:variable name="attachmentName"     select="@name" />
          <xsl:variable name="attachmentFileName" select="concat($parentType, $attachmentId, '_', $attachmentName)" />

          <xsl:choose>
            <xsl:when test="contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.PNG') or
                            contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.GIF') or
                            contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.JPG') or
                            contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.JPEG') or
                            contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.TIF') or
                            contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.TIFF') or
                            contains(translate($attachmentName,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'.BMP')">      
              <img max-width="200px" max-heigth="200px">
                <xsl:attribute name="src">
                  <xsl:value-of select="concat('resources/', $attachmentFileName)" />
                </xsl:attribute>
              </img>
              <br/>
              <!--
              <a target="blank">
                <xsl:attribute name="href">
                  <xsl:value-of select="concat('resources/', $attachmentFileName)" />
                </xsl:attribute>
                <xsl:value-of select="$attachmentName" />
              </a>
              <br/>
              -->              
            </xsl:when>
            <xsl:otherwise>
              &#160;&#160;
              <img>
                <xsl:attribute name="src">
                  <xsl:value-of select="$attachmentIcon" />
                </xsl:attribute>
              </img>
              <a target="blank">
                <xsl:attribute name="href">
                  <xsl:value-of select="concat('resources/', $attachmentFileName)" />
                </xsl:attribute>
                <xsl:value-of select="$attachmentName" />
              </a>
              <br/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:for-each>
        <br/>
      </xsl:if>
  </xsl:template>






  <!-- ======================= GENERATE [TD] METRICS ======================= -->

  <xsl:template name="generateNbTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItems" select="count($nodeSet)"/>

      <xsl:if test="$nbItems > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems"/></td></xsl:if>
      <xsl:if test="$nbItems = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

  <xsl:template name="generate3StatusTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsNew"          select="count($nodeSet[@status='10'])"/>
      <xsl:variable name="nbItemsAcknowledged" select="count($nodeSet[@status='20'])"/>
      <xsl:variable name="nbItemsApproved"     select="count($nodeSet[@status='30'])"/>

      <xsl:if test="$nbItemsNew > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsNew"/></td></xsl:if>
      <xsl:if test="$nbItemsNew = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsAcknowledged > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsAcknowledged"/></td></xsl:if>
      <xsl:if test="$nbItemsAcknowledged = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsApproved > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsApproved"/></td></xsl:if>
      <xsl:if test="$nbItemsApproved = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

  <xsl:template name="generate5StatusTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsNew"          select="count($nodeSet[@status='10'])"/>
      <xsl:variable name="nbItemsAssigned"     select="count($nodeSet[@status='20'])"/>
      <xsl:variable name="nbItemsAcknowledged" select="count($nodeSet[@status='30'])"/>
      <xsl:variable name="nbItemsResolved"     select="count($nodeSet[@status='40'])"/>
      <xsl:variable name="nbItemsClosed"       select="count($nodeSet[@status='50'])"/>

      <xsl:if test="$nbItemsNew > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsNew"/></td></xsl:if>
      <xsl:if test="$nbItemsNew = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsAssigned > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_orange</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsAssigned"/></td></xsl:if>
      <xsl:if test="$nbItemsAssigned = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_orange</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsAcknowledged > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsAcknowledged"/></td></xsl:if>
      <xsl:if test="$nbItemsAcknowledged = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsResolved > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsResolved"/></td></xsl:if>
      <xsl:if test="$nbItemsResolved = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsClosed > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsClosed"/></td></xsl:if>
      <xsl:if test="$nbItemsClosed = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>
                                          
  <xsl:template name="generate5StatusOnlyActiveTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsNew"          select="count($nodeSet[@status='10'])"/>
      <xsl:variable name="nbItemsAssigned"     select="count($nodeSet[@status='20'])"/>
      <xsl:variable name="nbItemsAcknowledged" select="count($nodeSet[@status='30'])"/>
      <xsl:variable name="nbItemsResolved"     select="count($nodeSet[@status='40'])"/>
      <xsl:variable name="nbItemsClosed"       select="count($nodeSet[@status='50'])"/>

      <xsl:if test="$nbItemsNew > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsNew"/></td></xsl:if>
      <xsl:if test="$nbItemsNew = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsAssigned > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_orange</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsAssigned"/></td></xsl:if>
      <xsl:if test="$nbItemsAssigned = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_orange</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsAcknowledged > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsAcknowledged"/></td></xsl:if>
      <xsl:if test="$nbItemsAcknowledged = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>
      <!--
      <xsl:if test="$nbItemsResolved > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsResolved"/></td></xsl:if>
      <xsl:if test="$nbItemsResolved = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsClosed > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsClosed"/></td></xsl:if>
      <xsl:if test="$nbItemsClosed = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute>&#160;</td></xsl:if>
      -->
  </xsl:template>






  <xsl:template name="generateTestResultTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsSuccess" select="count($nodeSet[@status='1'])"/>
      <xsl:variable name="nbItemsFailure" select="count($nodeSet[@status='2'])"/>
      <xsl:variable name="nbItemsUnknown" select="count($nodeSet[@status='4'])"/>

      <xsl:if test="$nbItemsSuccess > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsSuccess"/></td></xsl:if>
      <xsl:if test="$nbItemsSuccess = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsFailure > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsFailure"/></td></xsl:if>
      <xsl:if test="$nbItemsFailure = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsUnknown > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsUnknown"/></td></xsl:if>
      <xsl:if test="$nbItemsUnknown = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

  <xsl:template name="generateTestcaseResultTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsSuccess"     select="count($nodeSet[@status='1'])"/>
      <xsl:variable name="nbItemsFailure"     select="count($nodeSet[@status='2'])"/>
      <xsl:variable name="nbItemsNotExecuted" select="count($nodeSet[@status='0'])"/>
      <xsl:variable name="nbItemsRelative"    select="count($nodeSet[@status='3'])"/>

      <xsl:if test="$nbItemsSuccess > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsSuccess"/></td></xsl:if>
      <xsl:if test="$nbItemsSuccess = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsFailure > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsFailure"/></td></xsl:if>
      <xsl:if test="$nbItemsFailure = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsNotExecuted > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsNotExecuted"/></td></xsl:if>
      <xsl:if test="$nbItemsNotExecuted = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsRelative > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsRelative"/></td></xsl:if>
      <xsl:if test="$nbItemsRelative = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>


  <xsl:template name="generate3PriorityTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsHigh"   select="count($nodeSet[@priority='10'])"/>
      <xsl:variable name="nbItemsNormal" select="count($nodeSet[@priority='20'])"/>
      <xsl:variable name="nbItemsLow"    select="count($nodeSet[@priority='30'])"/>

      <xsl:if test="$nbItemsHigh > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsHigh"/></td></xsl:if>
      <xsl:if test="$nbItemsHigh = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsNormal > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsNormal"/></td></xsl:if>
      <xsl:if test="$nbItemsNormal = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsLow > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsLow"/></td></xsl:if>
      <xsl:if test="$nbItemsLow = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

  <xsl:template name="generate11PriorityTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItems0" select="count($nodeSet[@priority='0'])"/>
      <xsl:variable name="nbItems1" select="count($nodeSet[@priority='1'])"/>
      <xsl:variable name="nbItems2" select="count($nodeSet[@priority='2'])"/>
      <xsl:variable name="nbItems3" select="count($nodeSet[@priority='3'])"/>
      <xsl:variable name="nbItems4" select="count($nodeSet[@priority='4'])"/>
      <xsl:variable name="nbItems5" select="count($nodeSet[@priority='5'])"/>
      <xsl:variable name="nbItems6" select="count($nodeSet[@priority='6'])"/>
      <xsl:variable name="nbItems7" select="count($nodeSet[@priority='7'])"/>
      <xsl:variable name="nbItems8" select="count($nodeSet[@priority='8'])"/>
      <xsl:variable name="nbItems9" select="count($nodeSet[@priority='9'])"/>
      <xsl:variable name="nbItems10" select="count($nodeSet[@priority='10'])"/>

      <xsl:if test="$nbItems0 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems0"/></td></xsl:if>
      <xsl:if test="$nbItems0 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems1 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems1"/></td></xsl:if>
      <xsl:if test="$nbItems1 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems2 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems2"/></td></xsl:if>
      <xsl:if test="$nbItems2 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems3 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems3"/></td></xsl:if>
      <xsl:if test="$nbItems3 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems4 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems4"/></td></xsl:if>
      <xsl:if test="$nbItems4 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems5 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems5"/></td></xsl:if>
      <xsl:if test="$nbItems5 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems6 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems6"/></td></xsl:if>
      <xsl:if test="$nbItems6 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems7 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems7"/></td></xsl:if>
      <xsl:if test="$nbItems7 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems8 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems8"/></td></xsl:if>
      <xsl:if test="$nbItems8 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems9 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems9"/></td></xsl:if>
      <xsl:if test="$nbItems9 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItems10 > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItems10"/></td></xsl:if>
      <xsl:if test="$nbItems10 = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

  <xsl:template name="generate5SeverityTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItemsBlocking"    select="count($nodeSet[@severity='10'])"/>
      <xsl:variable name="nbItemsMajor"       select="count($nodeSet[@severity='20'])"/>
      <xsl:variable name="nbItemsMinor"       select="count($nodeSet[@severity='30'])"/>
      <xsl:variable name="nbItemsCosmetic"    select="count($nodeSet[@severity='40'])"/>
      <xsl:variable name="nbItemsEnhancement" select="count($nodeSet[@severity='50'])"/>

      <xsl:if test="$nbItemsBlocking > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsBlocking"/></td></xsl:if>
      <xsl:if test="$nbItemsBlocking = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsMajor > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_orange</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsMajor"/></td></xsl:if>
      <xsl:if test="$nbItemsMajor = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_orange</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsMinor > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsMinor"/></td></xsl:if>
      <xsl:if test="$nbItemsMinor = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_blue</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsCosmetic > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsCosmetic"/></td></xsl:if>
      <xsl:if test="$nbItemsCosmetic = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsEnhancement > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsEnhancement"/></td></xsl:if>
      <xsl:if test="$nbItemsEnhancement = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_grey</xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

  <xsl:template name="generateCompletionTd">
      <xsl:param name="nodeSet"/>
      <xsl:param name="tdStyle"/>

      <xsl:variable name="nbItems"        select="count($nodeSet)"/>
      <xsl:variable name="nbItemsNotImpl" select="count($nodeSet[@devCompleted='0'])"/>
      <xsl:variable name="nbItemsImpl"    select="count($nodeSet[@devCompleted='1'])"/>

      <xsl:if test="$nbItemsNotImpl > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsNotImpl"/></td></xsl:if>
      <xsl:if test="$nbItemsNotImpl = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_red</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsImpl > 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute><xsl:value-of disable-output-escaping="yes" select="$nbItemsImpl"/></td></xsl:if>
      <xsl:if test="$nbItemsImpl = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" />_green</xsl:attribute>&#160;</td></xsl:if>

      <xsl:if test="$nbItemsImpl > 0">
        <td width="150px" class="table_data_content">
          <!--<xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>-->
          <xsl:call-template name="percentage">
            <xsl:with-param name="max" select="$nbItems"/>
            <xsl:with-param name="value" select="$nbItemsImpl"/> 
          </xsl:call-template>
        </td>
      </xsl:if>
      <xsl:if test="$nbItemsImpl = 0"><td><xsl:attribute name="class"><xsl:value-of select="$tdStyle" /></xsl:attribute>&#160;</td></xsl:if>
  </xsl:template>

<!-- ============================= DEFECTS ================================= -->

  <xsl:template name="generateUniqueDefectsDetailsFromAllLevels">

    <!-- we use this to search for all distinct defects in the document -->
    <xsl:key name="distinct-defects" match="defect" use="@id"/>
    <xsl:variable name="nodeSet" select="//defect[generate-id()=generate-id(key('distinct-defects',@id))]" />

    <xsl:call-template name="generateDefectsDetails">
      <xsl:with-param name="nodeSet" select="$nodeSet"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateDefectsDetailsFromCurrentLevel">
    <xsl:call-template name="generateDefectsDetails">
      <xsl:with-param name="nodeSet" select="defect"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateOnlyActiveDefectsDetailsFromCurrentLevel">
    <xsl:call-template name="generateDefectsDetails">
      <xsl:with-param name="nodeSet" select="defect[@status='10' or @status='20' or @status='30']"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateDefectsDetails">
    <xsl:param name="nodeSet"/>

    <xsl:for-each select="$nodeSet">
      <xsl:call-template name="generateDefectDetails">
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="generateDefectDetails">
      <xsl:variable name="defectId"                    select="@id" />
      <xsl:variable name="defectName"                  select="@name" />
      <xsl:variable name="defectPriority"              select="@priority" />
      <xsl:variable name="defectSeverity"              select="@severity" />
      <xsl:variable name="defectReporter"              select="@reporter" />
      <xsl:variable name="defectStatus"                select="@status" />
      <xsl:variable name="defectReproducibility"       select="@reproducibility" />
      <xsl:variable name="defectFixTargetDate"         select="@fix_target_date" />
      <xsl:variable name="defectFixTargetVersion"      select="@fix_target_version" />
      <xsl:variable name="defectCompletionPercentage"  select="@completion_percentage" />
      <xsl:variable name="defectFixType"               select="@fix_type" />
      <xsl:variable name="defectFixPatches"            select="@fix_patches" />
      <xsl:variable name="defectPlatformName"          select="@platform_name" />
      <xsl:variable name="defectOsName"                select="@os_name" />
      <xsl:variable name="defectDescription"           select="description" />
      <xsl:variable name="defectStepToReproduce"       select="stepToReproduce" />
      <xsl:variable name="defectFixDescription"        select="fixDescription" />
      <xsl:variable name="defectExternalId"            select="@externalId" />
      <xsl:variable name="defectUrl"                   select="@url" />

      <!-- anchor link -->
      <xsl:variable name="defectAnchor" select="concat('D_', $defectId)" />
      <a>
        <xsl:attribute name="name">
          <!--<xsl:value-of select="$defectAnchor" />-->
          <xsl:call-template name="generateUniqueId">
            <xsl:with-param name="type" select="'defect'"/>
            <xsl:with-param name="id" select="@id"/>
          </xsl:call-template>
        </xsl:attribute>
      </a>

      <!-- BUGTRACKING_TYPE_INTEGRATED -->
      <xsl:if test="$bugtrackingType = 1">
        <table width="100%">
          <tr>
            <td colspan="4" class="table_data_header3">
              <xsl:call-template name="urlToObject">
                <xsl:with-param name="globalId" select="concat('D_', $defectId)"/>
                <xsl:with-param name="name"     select="$defectName"/> 
              </xsl:call-template>
            </td>
          </tr>

          <tr>
              <td width="25%" class="table_data_subheader">#defect.Status#</td>
              <td width="25%" class="table_data_subheader">#defect.Priority#</td>
              <td width="25%" class="table_data_subheader">#defect.Severity#</td>
              <td width="25%" class="table_data_subheader">#defect.Reproducibility#</td>
          </tr>
          <tr>
            <td class="table_data_content">
              <xsl:call-template name="defectStatusToString"><xsl:with-param name="status" select="$defectStatus"/></xsl:call-template>
            </td>
            <td class="table_data_content">
              <xsl:call-template name="defectPriorityToString"><xsl:with-param name="priority" select="$defectPriority"/></xsl:call-template>
            </td>
            <td class="table_data_content">
              <xsl:call-template name="defectSeverityToString"><xsl:with-param name="severity" select="$defectSeverity"/></xsl:call-template>
            </td>
            <td class="table_data_content">
              <xsl:call-template name="defectReproducibilityToString"><xsl:with-param name="reproducibility" select="$defectReproducibility"/></xsl:call-template>
            </td>
          </tr>

          <tr>
              <td width="25%" class="table_data_subheader">#defect.Fix_Target_Date#</td>
              <td width="25%" class="table_data_subheader">#defect.Fix_Target_Version#</td>
              <td width="25%" class="table_data_subheader">#defect.Completion_Percentage#</td>
              <td width="25%" class="table_data_subheader">#defect.Fix_Type#</td>
          </tr>
          <tr>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectFixTargetDate"/>
            </td>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectFixTargetVersion"/>
            </td>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectCompletionPercentage"/>%
            </td>
            <td class="table_data_content">
              <xsl:call-template name="defectFixTypeToString"><xsl:with-param name="fixType" select="$defectFixType"/></xsl:call-template>
            </td>
          </tr>
          <tr>
              <td width="25%" class="table_data_subheader">#defect.Fix_Description#</td>
              <td width="25%" class="table_data_subheader">#defect.Fix_Patches#</td>
              <td width="25%" class="table_data_subheader">#defect.Platform_Name#</td>
              <td width="25%" class="table_data_subheader">#defect.Os_Name#</td>
          </tr>
          <tr>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectFixDescription"/>
            </td>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectFixPatches"/>
            </td>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectPlatformName"/>
            </td>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectOsName"/>
            </td>
          </tr>
          <tr>
              <td width="25%" class="table_data_subheader">#defect.Reporter#</td>
              <td width="25%" class="table_data_subheader">#defect.Assigned_to#</td>
              <td width="25%" class="table_data_subheader">#defect.Found_in#</td>
              <td width="25%" class="table_data_subheader">#defect.Fixed_in#</td>
          </tr>
          <tr>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectReporter"/>
            </td>
            <td class="table_data_content">
              <xsl:for-each select="assigned_to">
                <xsl:value-of disable-output-escaping="yes" select="@name"/>&#160;&#160;
              </xsl:for-each>              
            </td>
            <td class="table_data_content">
				<xsl:for-each select="foundIn/sut">
				  <xsl:call-template name="generateVeryShortSut">
				  </xsl:call-template>
				  <br/>
				</xsl:for-each>             
            </td>
            <td class="table_data_content">
				<xsl:for-each select="fixedIn/sut">
				  <xsl:call-template name="generateVeryShortSut">
				  </xsl:call-template>
				  <br/>				
				</xsl:for-each>
            </td>
          </tr>
		  
  

          <!-- description is mandatory so displayed even if empty -->
          <tr><td colspan="4" class="table_data_subheader">#defect.Description#</td></tr>
          <tr>
            <td colspan="4" class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$defectDescription" />
              &#160; <!-- to avoid having a ugly effect when there is no description -->
              <br/><br/>
              <xsl:call-template name="generateAttachments">
                <xsl:with-param name="parentType" select="'D_'"/>
                <xsl:with-param name="attachmentIcon" select="'resources/images/IconAttachment.gif'"/>
              </xsl:call-template>              
            </td>
          </tr>

          <xsl:if test="string-length($defectStepToReproduce)>0">
            <tr><td colspan="4" class="table_data_subheader">#defect.Step_To_Reproduce#</td></tr>
            <tr>
              <td colspan="4" class="table_data_content">
                <xsl:value-of disable-output-escaping="yes" select="$defectStepToReproduce" />
              </td>
            </tr>
          </xsl:if>
      
          <!-- custom fields -->
          <xsl:call-template name="insertCustomFields">
            <xsl:with-param name="colSpan" select="'4'"/>
          </xsl:call-template>
        </table>
		
      </xsl:if>
      <!-- BUGTRACKING_TYPE_GENERIC -->
      <xsl:if test="$bugtrackingType = 2">
        <table width="100%">
          <tr>
              <td width="30%" class="table_data_subheader">#defect.Id#</td>
              <td width="70%" class="table_data_subheader">#defect.External_id#</td>
          </tr>
          <tr>
            <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$defectId" /></td>
            <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$defectExternalId" /></td>
          </tr>
        </table>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_MANTIS -->
      <!-- BUGTRACKING_TYPE_BUGZILLA -->
      <!-- BUGTRACKING_TYPE_TRAC -->
      <!-- BUGTRACKING_TYPE_JIRA -->
      <xsl:if test="$bugtrackingType = 3 or $bugtrackingType = 4 or $bugtrackingType = 5 or $bugtrackingType = 6">
        <table width="100%">
          <tr>
              <td width="30%" class="table_data_subheader">#defect.External_id#</td>
              <td width="40%" class="table_data_subheader">#defect.Link#</td>
          </tr>
          <tr>
            <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$defectId" /></td>
            <td class="table_data_content">
              <!-- anchor link -->
              <a>
                <xsl:attribute name="href">
                  <xsl:value-of select="$defectUrl" />
                </xsl:attribute>
                <xsl:value-of select="$defectUrl" />
              </a>
            </td>
          </tr>
        </table>
      </xsl:if>
      <br/>
  </xsl:template>

  <xsl:template name="generateShortDefectHeader">
      <!-- BUGTRACKING_TYPE_INTEGRATED -->
      <xsl:if test="$bugtrackingType = 1">
        <tr>
          <td width="55%" class="table_data_subheader">#report.Defects#</td>
          <td width="15%" class="table_data_subheader">#defect.Status#</td>
          <td width="15%" class="table_data_subheader">#defect.Priority#</td>
          <td width="15%" class="table_data_subheader">#defect.Severity#</td>
        </tr>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_GENERIC -->
      <xsl:if test="$bugtrackingType = 2">
        <tr>
          <td width="30%" class="table_data_subheader">#report.Defects#</td>
          <td width="70%" class="table_data_subheader">#defect.External_id#</td>
        </tr>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_MANTIS -->
      <!-- BUGTRACKING_TYPE_BUGZILLA -->
      <!-- BUGTRACKING_TYPE_TRAC -->
      <!-- BUGTRACKING_TYPE_JIRA -->
      <xsl:if test="$bugtrackingType = 3 or $bugtrackingType = 4 or $bugtrackingType = 5 or $bugtrackingType = 6">
        <tr>
          <td width="30%" class="table_data_subheader">#defect.External_id#</td>
          <td width="40%" class="table_data_subheader">#defect.Link#</td>
        </tr>
      </xsl:if>
  </xsl:template>
  
  <xsl:template name="generateShortDefectSubHeader">
      <xsl:param name="category"/> <!-- found in or fixed in -->
      
      <!-- BUGTRACKING_TYPE_INTEGRATED -->
      <xsl:if test="$bugtrackingType = 1">
        <tr><td class="table_data_subsubheader" colspan="4"><xsl:value-of select="$category" /></td></tr>
        <tr>
          <td width="55%" class="table_data_subsubheader">#report.Defects#</td>
          <td width="15%" class="table_data_subsubheader">#defect.Status#</td>
          <td width="15%" class="table_data_subsubheader">#defect.Priority#</td>
          <td width="15%" class="table_data_subsubheader">#defect.Severity#</td>
        </tr>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_GENERIC -->
      <xsl:if test="$bugtrackingType = 2">
        <tr><td class="table_data_subsubheader" colspan="2"><xsl:value-of select="$category" /></td></tr>
        <tr>
          <td width="30%" class="table_data_subsubheader">#report.Defects#</td>
          <td width="70%" class="table_data_subsubheader">#defect.External_id#</td>
        </tr>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_MANTIS -->
      <!-- BUGTRACKING_TYPE_BUGZILLA -->
      <!-- BUGTRACKING_TYPE_TRAC -->
      <!-- BUGTRACKING_TYPE_JIRA -->
      <xsl:if test="$bugtrackingType = 3 or $bugtrackingType = 4 or $bugtrackingType = 5 or $bugtrackingType = 6">
        <tr><td class="table_data_subsubheader" colspan="2"><xsl:value-of select="$category" /></td></tr>
        <tr>
          <td width="30%" class="table_data_subsubheader">#defect.External_id#</td>
          <td width="40%" class="table_data_subsubheader">#defect.Link#</td>
        </tr>
      </xsl:if>
  </xsl:template>




  <xsl:template name="generateShortDefect">
      <xsl:variable name="defectId"                    select="@id" />
      <xsl:variable name="defectName"                  select="@name" />

      <xsl:variable name="defectPriority"              select="@priority" />
      <xsl:variable name="defectSeverity"              select="@severity" />
      <xsl:variable name="defectStatus"                select="@status" />

      <xsl:variable name="defectExternalId"            select="@externalId" />
      <xsl:variable name="defectUrl"                   select="@url" />


      <!-- BUGTRACKING_TYPE_INTEGRATED -->
      <xsl:if test="$bugtrackingType = 1">
        <tr>
          <td class="table_data_content">
              <!-- anchor link -->
              <xsl:variable name="defectSharp" select="concat('#D_', $defectId)" />
              <a>
                <xsl:attribute name="href">
                  <!--<xsl:value-of select="$defectSharp" />-->
                  <xsl:call-template name="generateSharpUniqueId">
                    <xsl:with-param name="type" select="'defect'"/>
                    <xsl:with-param name="id" select="@id"/>
                  </xsl:call-template>
                </xsl:attribute>
                [D_<xsl:value-of select="$defectId"/>]&#160;<xsl:value-of select="$defectName" />
              </a>
          </td>
          <td class="table_data_content">
            <xsl:call-template name="defectStatusToString"><xsl:with-param name="status" select="$defectStatus"/></xsl:call-template>
          </td>
          <td class="table_data_content">
            <xsl:call-template name="defectPriorityToString"><xsl:with-param name="priority" select="$defectPriority"/></xsl:call-template>
          </td>
          <td class="table_data_content">
            <xsl:call-template name="defectSeverityToString"><xsl:with-param name="severity" select="$defectSeverity"/></xsl:call-template>
          </td>
        </tr>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_GENERIC -->
      <xsl:if test="$bugtrackingType = 2">
        <tr>
          <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$defectId" /></td>
          <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$defectExternalId" /></td>
        </tr>
      </xsl:if>
      <!-- BUGTRACKING_TYPE_MANTIS -->
      <!-- BUGTRACKING_TYPE_BUGZILLA -->
      <!-- BUGTRACKING_TYPE_TRAC -->
      <!-- BUGTRACKING_TYPE_JIRA -->
      <xsl:if test="$bugtrackingType = 3 or $bugtrackingType = 4 or $bugtrackingType = 5 or $bugtrackingType = 6">
        <tr>
          <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$defectId" /></td>
          <td class="table_data_content">
            <!-- anchor link -->
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="$defectUrl" />
              </xsl:attribute>
              <xsl:value-of select="$defectUrl" />
            </a>
          </td>
        </tr>
      </xsl:if>
  </xsl:template>

  <xsl:template name="defectPriorityToString">
    <xsl:param name="priority"/>
    <xsl:if test="$priority='10'"><span class="text_red">#defect.priority.High#</span></xsl:if>
    <xsl:if test="$priority='20'"><span class="text_blue">#defect.priority.Normal#</span></xsl:if>
    <xsl:if test="$priority='30'"><span class="text_grey">#defect.priority.Low#</span></xsl:if>
  </xsl:template>

  <xsl:template name="defectSeverityToString">
    <xsl:param name="severity"/>
    <xsl:if test="$severity='10'"><span class="text_red">#defect.severity.Blocking#</span></xsl:if>
    <xsl:if test="$severity='20'"><span class="text_orange">#defect.severity.Major#</span></xsl:if>
    <xsl:if test="$severity='30'"><span class="text_blue">#defect.severity.Minor#</span></xsl:if>
    <xsl:if test="$severity='40'"><span class="text_green">#defect.severity.Cosmetic#</span></xsl:if>
    <xsl:if test="$severity='50'"><span class="text_grey">#defect.severity.Enhancement#</span></xsl:if>
  </xsl:template>

  <xsl:template name="defectStatusToString">
    <xsl:param name="status"/>
    <xsl:if test="$status='10'"><span class="text_red">#defect.status.New#</span></xsl:if>
    <xsl:if test="$status='20'"><span class="text_orange">#defect.status.Assigned#</span></xsl:if>
    <xsl:if test="$status='30'"><span class="text_blue">#defect.status.Acknowledged#</span></xsl:if>
    <xsl:if test="$status='40'"><span class="text_green">#defect.status.Resolved#</span></xsl:if>
    <xsl:if test="$status='50'"><span class="text_grey">#defect.status.Closed#</span></xsl:if>
  </xsl:template>

  <xsl:template name="defectReproducibilityToString">
    <xsl:param name="reproducibility"/>
    <xsl:if test="$reproducibility='10'">#defect.reproducibility.Always#</xsl:if>
    <xsl:if test="$reproducibility='20'">#defect.reproducibility.Random#</xsl:if>
    <xsl:if test="$reproducibility='30'">#defect.reproducibility.Once#</xsl:if>
  </xsl:template>

  <xsl:template name="defectFixTypeToString">
    <xsl:param name="fixType"/>
    <xsl:if test="$fixType='10'">#defect.correction_type.Resolved#</xsl:if>
    <xsl:if test="$fixType='20'">#defect.correction_type.Unable_to_reproduce#</xsl:if>
    <xsl:if test="$fixType='30'">#defect.correction_type.No_change_required#</xsl:if>
    <xsl:if test="$fixType='40'">#defect.correction_type.Duplicate#</xsl:if>
  </xsl:template>

  <xsl:template name="generateListOfUniqueIds">
      <xsl:param name="nodeSet"/>
      <xsl:param name="prefix"/>

      <xsl:variable name="nbItems" select="count($nodeSet)"/>

      <xsl:if test="$nbItems > 0">
        <td class="table_data_subheader_bg_green">
          <xsl:for-each select="$nodeSet">
            [<xsl:value-of select="$prefix"/><xsl:value-of select="@id"/>]&#160;<xsl:value-of select="@name"/><br/>
          </xsl:for-each>
        </td>
      </xsl:if>

      <xsl:if test="$nbItems = 0">
        <td class="table_data_subheader_bg_red">-</td>
      </xsl:if>
  </xsl:template>


<!-- ============================== SUTS =================================== -->
  <xsl:template name="generateSutsDetailsFromCurrentLevel">
    <xsl:call-template name="generateSutsDetails">
      <xsl:with-param name="nodeSet" select="sut"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateSutsDetails">
    <xsl:param name="nodeSet"/>
    <xsl:for-each select="$nodeSet">
      <xsl:call-template name="generateSutDetails"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="generateSutDetails">
    <xsl:variable name="nbRequirements"     select="count(requirement)"/>
    <xsl:variable name="nbFoundInDefects"   select="count(foundIn/defect)"/>
    <xsl:variable name="nbFixedInDefects"   select="count(fixedIn/defect)"/>
    <xsl:variable name="nbCampaignSessions" select="count(campaignsession)"/>

    <xsl:variable name="sutId"      select="@id" />
    <xsl:variable name="sutName"    select="@name" />
    <xsl:variable name="sutVersion" select="@version" />
 
    <xsl:if test="$nbRequirements > 0 or $nbFoundInDefects > 0 or $nbFixedInDefects > 0 or $nbCampaignSessions > 0">
      <a>
        <xsl:attribute name="name">
          <xsl:call-template name="generateUniqueId">
            <xsl:with-param name="type" select="'sut'"/>
            <xsl:with-param name="id" select="@id"/>
          </xsl:call-template>
        </xsl:attribute>
      </a>
      <h3>
        <img src="resources/images/IconSut.gif"/>&#160;
        <xsl:call-template name="urlToObject">
          <xsl:with-param name="globalId" select="concat('SU_', $sutId)"/>
          <xsl:with-param name="name"     select="concat($sutName, ' ', $sutVersion)"/> 
        </xsl:call-template>
        &#160;
      </h3>


      <xsl:if test="$specificationInLifecycle = 'true'">
        <!-- REQUIREMENTS / SPECIFICATIONS / TESTS -->
        <xsl:if test="$nbRequirements > 0">
          <table border="1px" width="100%" cellpadding="0">
            <tr>
              <td class="table_data_subsubheader" width="33%">#sut.Requirements#</td>
              <td class="table_data_subsubheader" width="66%">
                <table border="0px" width="100%">
                  <tr>         
                    <td class="table_data_subsubheader innertable innertable_firstLine" width="50%">#sut.Specifications#</td>
                    <td class="table_data_subsubheader innertable innertable_firstLine innertable_lastColumn" width="50%">#sut.Tests#</td>
                  </tr>
                </table>
              </td>
            </tr>
  
            <xsl:for-each select="requirement">
              <tr>
                <!-- *********** FIRST COLUMN *********** -->
                <td class="table_data_content">
                  <!-- anchor link -->
                  <a>
                    <xsl:attribute name="href">
                      <xsl:call-template name="generateSharpUniqueId">
                        <xsl:with-param name="type" select="'requirement'"/>
                        <xsl:with-param name="id" select="@id"/>
                      </xsl:call-template>
                    </xsl:attribute>
                    [R_<xsl:value-of select="@id"/>]&#160;<xsl:value-of select="@name"/>
                  </a>
                </td>
  
                <xsl:variable name="nbSpecifications" select="count(specification)"/>
                <xsl:if test="$nbSpecifications > 0">
                  <!-- *********** SECOND COLUMN *********** -->
                  <td class="table_data_content">
  
                    <table border="0px" width="100%">
                    <xsl:for-each select="specification">
                      <tr>
                        <td width="50%">
                          <xsl:attribute name="class">
                            table_data_subheader_bg_green innertable
                            <xsl:if test="position()=1">
                              innertable_firstLine
                            </xsl:if>
                          </xsl:attribute>
                          <!-- anchor link -->
                          <a>
                            <xsl:attribute name="href">
                              <xsl:call-template name="generateSharpUniqueId">
                                <xsl:with-param name="type" select="'specification'"/>
                                <xsl:with-param name="id" select="@id"/>
                              </xsl:call-template>
                            </xsl:attribute>
                            [S_<xsl:value-of select="@id"/>]&#160;<xsl:value-of select="@name"/>
                          </a>
                        </td>
  
                        <xsl:variable name="nbTests" select="count(test)"/>
                        <xsl:if test="$nbTests > 0">
                          <!-- *********** THIRD COLUMN *********** -->
                          <td class="table_data_subheader_bg_green innertable innertable_firstLine innertable_lastColumn" width="50%">
  
                            <xsl:for-each select="test">
                                  <a>
                                    <xsl:attribute name="href">
                                      <xsl:call-template name="generateSharpUniqueId">
                                        <xsl:with-param name="type" select="'test'"/>
                                        <xsl:with-param name="id" select="@id"/>
                                      </xsl:call-template>
                                    </xsl:attribute>
                                    [T_<xsl:value-of select="@id"/>]&#160;<xsl:value-of select="@name"/>
                                  </a>
                                  <br/>
                            </xsl:for-each>
  
                          </td>
                        </xsl:if> <!-- nb tests > 0 -->
  
                        <xsl:if test="$nbTests = 0">
                          <td class="table_data_subheader_bg_red innertable innertable_firstLine innertable_lastColumn" width="50%">
                            -
                          </td>
                        </xsl:if>
  
                      </tr>
                    </xsl:for-each> <!-- for each specification -->
                    </table>
                  </td>
                </xsl:if> <!-- nb specifications > 0 -->
  
                <xsl:if test="$nbSpecifications = 0">
                  <td class="table_data_subheader_bg_red innertable" width="50%">
                    -
                  </td>
                </xsl:if>
  
              </tr>
            </xsl:for-each> <!-- for each requirement -->
          </table>
          <br/>
        </xsl:if> <!-- nb requirements > 0 -->
      </xsl:if>
      
      <xsl:if test="$specificationInLifecycle = 'false'">
        <!-- REQUIREMENTS / TESTS -->
        <xsl:if test="$nbRequirements > 0">
          <table border="1px" width="100%" cellpadding="0">
            <tr>
              <td class="table_data_subsubheader" width="33%">#sut.Requirements#</td>
              <td class="table_data_subsubheader" width="66%">#sut.Tests#</td>
            </tr>
  
            <xsl:for-each select="requirement">
              <tr>
                <!-- *********** FIRST COLUMN *********** -->
                <td class="table_data_content">
                  <!-- anchor link -->
                  <a>
                    <xsl:attribute name="href">
                      <xsl:call-template name="generateSharpUniqueId">
                        <xsl:with-param name="type" select="'requirement'"/>
                        <xsl:with-param name="id" select="@id"/>
                      </xsl:call-template>
                    </xsl:attribute>
                    [R_<xsl:value-of select="@id"/>]&#160;<xsl:value-of select="@name"/>
                  </a>
                </td>

                <xsl:variable name="nbTests" select="count(test)"/>
                <xsl:if test="$nbTests > 0">
                  <!-- *********** THIRD COLUMN *********** -->
                  <td class="table_data_subheader_bg_green innertable innertable_firstLine innertable_lastColumn" width="50%">

                    <xsl:for-each select="test">
                          <a>
                            <xsl:attribute name="href">
                              <xsl:call-template name="generateSharpUniqueId">
                                <xsl:with-param name="type" select="'test'"/>
                                <xsl:with-param name="id" select="@id"/>
                              </xsl:call-template>
                            </xsl:attribute>
                            [T_<xsl:value-of select="@id"/>]&#160;<xsl:value-of select="@name"/>
                          </a>
                          <br/>
                    </xsl:for-each>

                  </td>
                </xsl:if> <!-- nb tests > 0 -->

                <xsl:if test="$nbTests = 0">
                  <td class="table_data_subheader_bg_red innertable innertable_firstLine innertable_lastColumn" width="50%">
                    -
                  </td>
                </xsl:if>

              </tr>
            </xsl:for-each> <!-- for each requirement -->
          </table>
          <br/>
        </xsl:if> <!-- nb requirements > 0 -->
      </xsl:if>
      
      
      
      <xsl:if test="$nbFoundInDefects > 0 or $nbFixedInDefects > 0">
        <table width="100%">
          <!-- FOUND IN DEFECTS -->
          <xsl:if test="$nbFoundInDefects > 0">
            <xsl:call-template name="generateShortDefectSubHeader">
              <xsl:with-param name="category" select="'#sut.Bugs_Found#'"/>
            </xsl:call-template>
            <xsl:for-each select="foundIn/defect">
              <xsl:call-template name="generateShortDefect">
              </xsl:call-template>
            </xsl:for-each>
          </xsl:if>

          <!-- FIXED IN DEFECTS -->
          <xsl:if test="$nbFixedInDefects > 0">
            <xsl:call-template name="generateShortDefectSubHeader">
              <xsl:with-param name="category" select="'#sut.Bugs_Fixed#'"/>
            </xsl:call-template>
            <xsl:for-each select="fixedIn/defect">
              <xsl:call-template name="generateShortDefect">
              </xsl:call-template>
            </xsl:for-each>
          </xsl:if>

        </table>
        <br/>
      </xsl:if>

      <!-- CAMPAIGN SESSIONS -->
      <xsl:if test="$nbCampaignSessions > 0">
        <table width="100%">
        <tr>
          <td width="70%" class="table_data_subsubheader">#sut.Campaign_Sessions#</td>
          <td width="15%" class="table_data_subsubheader">#report.Start_date#</td>
          <td width="15%" class="table_data_subsubheader">#report.Stop_date#</td>
        </tr>
        <xsl:for-each select="campaignsession">
          <xsl:call-template name="generateShortCampaignSession"/>
        </xsl:for-each>
        </table>
        <br/>
      </xsl:if>

    </xsl:if> <!-- nb requirements > 0 or nb defects > 0 or nb sessions > 0 -->
  </xsl:template>

  
<xsl:template name="generateShortSut">
	<xsl:variable name="sutId"    select="@id" />
	<xsl:variable name="sutName"  select="@name" />

	<tr>
	  <td class="table_data_content">
		  <!-- anchor link -->
		  <xsl:variable name="defectSharp" select="concat('#SU_', $sutId)" />
		  <a>
			<xsl:attribute name="href">
			  <!--<xsl:value-of select="$defectSharp" />-->
			  <xsl:call-template name="generateSharpUniqueId">
				<xsl:with-param name="type" select="'sut'"/>
				<xsl:with-param name="id" select="@id"/>
			  </xsl:call-template>
			</xsl:attribute>
			[SU_<xsl:value-of select="$sutId"/>]&#160;&#160;<xsl:value-of select="$sutName" />
		  </a>
	  </td> 
	</tr>
</xsl:template>

<xsl:template name="generateVeryShortSut">
	<xsl:variable name="sutId"    select="@id" />
	<xsl:variable name="sutName"  select="@name" />

	[SU_<xsl:value-of select="$sutId"/>]&#160;&#160;<xsl:value-of select="$sutName" />
</xsl:template>

<!-- ========================== REQUIREMENTS =============================== -->
  <xsl:template name="requirementTypeToString">
    <xsl:param name="type"/>
    <xsl:if test="$type='10'">#requirement.globaltype.Functional#</xsl:if>
    <xsl:if test="$type='110'">#requirement.type.Reliability_Maturity#</xsl:if>
    <xsl:if test="$type='120'">#requirement.type.Reliability_FaultTolerance#</xsl:if>
    <xsl:if test="$type='130'">#requirement.type.Reliability_Recoverability#</xsl:if>
    <xsl:if test="$type='140'">#requirement.type.Reliability_Availability#</xsl:if>
    <xsl:if test="$type='150'">#requirement.type.Reliability_Degradability#</xsl:if>
    <xsl:if test="$type='160'">#requirement.type.Reliability_Accuracy#</xsl:if>
    <xsl:if test="$type='170'">#requirement.type.Reliability_Security#</xsl:if>
    <xsl:if test="$type='210'">#requirement.type.Usability_Accessibility#</xsl:if>
    <xsl:if test="$type='220'">#requirement.type.Usability_Understandability#</xsl:if>
    <xsl:if test="$type='230'">#requirement.type.Usability_Learnability#</xsl:if>
    <xsl:if test="$type='240'">#requirement.type.Usability_Operability#</xsl:if>
    <xsl:if test="$type='250'">#requirement.type.Usability_Explicitness#</xsl:if>
    <xsl:if test="$type='260'">#requirement.type.Usability_Customizability#</xsl:if>
    <xsl:if test="$type='270'">#requirement.type.Usability_Attractivity#</xsl:if>
    <xsl:if test="$type='280'">#requirement.type.Usability_Clarity#</xsl:if>
    <xsl:if test="$type='290'">#requirement.type.Usability_Helpfulness#</xsl:if>
    <xsl:if test="$type='300'">#requirement.type.Usability_Userfriendlyness#</xsl:if>
    <xsl:if test="$type='410'">#requirement.type.Efficiency_Time#</xsl:if>
    <xsl:if test="$type='420'">#requirement.type.Efficiency_Resources#</xsl:if>
    <xsl:if test="$type='510'">#requirement.type.Documentation#</xsl:if>
    <xsl:if test="$type='610'">#requirement.type.Maintainability_Analyzability#</xsl:if>
    <xsl:if test="$type='620'">#requirement.type.Maintainability_Extensibility#</xsl:if>
    <xsl:if test="$type='630'">#requirement.type.Maintainability_Stability#</xsl:if>
    <xsl:if test="$type='640'">#requirement.type.Maintainability_Testability#</xsl:if>
    <xsl:if test="$type='650'">#requirement.type.Maintainability_Manageability#</xsl:if>
    <xsl:if test="$type='660'">#requirement.type.Maintainability_Reusability#</xsl:if>
    <xsl:if test="$type='670'">#requirement.type.Maintainability_Supportability#</xsl:if>
    <xsl:if test="$type='680'">#requirement.type.Maintainability_Traceability#</xsl:if>
    <xsl:if test="$type='710'">#requirement.type.Portability_Compatibility#</xsl:if>
    <xsl:if test="$type='720'">#requirement.type.Portability_Adaptability#</xsl:if>
    <xsl:if test="$type='730'">#requirement.type.Portability_Installability#</xsl:if>
    <xsl:if test="$type='740'">#requirement.type.Portability_Conformance#</xsl:if>
    <xsl:if test="$type='750'">#requirement.type.Portability_Replaceability#</xsl:if>
    <xsl:if test="$type='760'">#requirement.type.Portability_Safety#</xsl:if>
    <xsl:if test="$type='770'">#requirement.type.Portability_Size#</xsl:if>
    <xsl:if test="$type='810'">#requirement.type.Organisation_Delivery#</xsl:if>
    <xsl:if test="$type='820'">#requirement.type.Organisation_Implementation#</xsl:if>
    <xsl:if test="$type='830'">#requirement.type.Organisation_Compliance#</xsl:if>
    <xsl:if test="$type='910'">#requirement.type.External_Interoperability#</xsl:if>
    <xsl:if test="$type='920'">#requirement.type.External_Legislative_Safety#</xsl:if>
    <xsl:if test="$type='930'">#requirement.type.External_Legislative_Privacy#</xsl:if>
    <xsl:if test="$type='940'">#requirement.type.External_Legislative_Licensing#</xsl:if>
    <xsl:if test="$type='950'">#requirement.type.External_Ethical#</xsl:if>
  </xsl:template>

  <xsl:template name="requirementPriorityToString">
    <xsl:param name="priority"/>
    <xsl:if test="$priority='10'"><span class="text_red">#requirement.priority.High#</span></xsl:if>
    <xsl:if test="$priority='20'"><span class="text_blue">#requirement.priority.Normal#</span></xsl:if>
    <xsl:if test="$priority='30'"><span class="text_grey">#requirement.priority.Low#</span></xsl:if>
  </xsl:template>

  <xsl:template name="requirementStatusToString">
    <xsl:param name="status"/>
    <xsl:if test="$status='10'"><span class="text_red">#requirement.status.New#</span></xsl:if>
    <xsl:if test="$status='20'"><span class="text_blue">#requirement.status.Acknowledged#</span></xsl:if>
    <xsl:if test="$status='30'"><span class="text_green">#requirement.status.Approved#</span></xsl:if>
  </xsl:template>

  <xsl:template name="generateUniqueRequirementsDetailsFromAllLevels">
    <!-- we use this to search for all distinct requirements in the document -->
    <xsl:key name="distinct-requirements" match="requirement" use="@id"/>
    <xsl:variable name="nodeSet" select="//requirement[generate-id()=generate-id(key('distinct-requirements',@id))]" />

    <xsl:call-template name="generateRequirementsDetails">
      <xsl:with-param name="nodeSet" select="$nodeSet"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateRequirementsDetailsFromCurrentLevel">
    <xsl:call-template name="generateRequirementsDetails">
      <xsl:with-param name="nodeSet" select="requirement"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateRequirementsDetails">
    <xsl:param name="nodeSet"/>

    <xsl:for-each select="$nodeSet">
      <xsl:call-template name="generateRequirementDetails"/>
    </xsl:for-each>
  </xsl:template>



  <xsl:template name="generateRequirementDetails">
    <xsl:variable name="requirementId"            select="@id" />
    <xsl:variable name="requirementName"          select="@name" />
    <xsl:variable name="requirementDescription"   select="description" />
    <xsl:variable name="requirementType"          select="@type" />
    <xsl:variable name="requirementPriority"      select="@priority" />
    <xsl:variable name="requirementStatus"        select="@status" />
    <xsl:variable name="requirementUrl"           select="@url" />

    <br/>
    <a>
      <xsl:attribute name="name">
        <xsl:call-template name="generateUniqueId">
          <xsl:with-param name="type" select="'requirement'"/>
          <xsl:with-param name="id" select="@id"/>
        </xsl:call-template>
      </xsl:attribute>
    </a>
    
    <table width="100%">
      <tr>
        <td colspan="3" class="table_data_subheader">
          <xsl:call-template name="urlToObject">
            <xsl:with-param name="globalId" select="concat('R_', $requirementId)"/>
            <xsl:with-param name="name"     select="$requirementName"/> 
          </xsl:call-template>
        </td>
      </tr>
      
      
      <!-- REQUIREMENT_TYPE_INTEGRATED -->
      <xsl:if test="$requirementTrackingType = 1">
        <tr>
          <td width="33%" class="table_data_subsubheader">#requirement.Type#</td>
          <td width="33%" class="table_data_subsubheader">#requirement.Priority#</td>
          <td width="33%" class="table_data_subsubheader">#requirement.Status#</td>
        </tr>
        <tr>
          <td class="table_data_content">
            <xsl:call-template name="requirementTypeToString"><xsl:with-param name="type" select="$requirementType"/></xsl:call-template>
          </td>
          <td class="table_data_content">
            <xsl:call-template name="requirementPriorityToString"><xsl:with-param name="priority" select="$requirementPriority"/></xsl:call-template>
          </td>
          <td class="table_data_content">
            <xsl:call-template name="requirementStatusToString"><xsl:with-param name="status" select="$requirementStatus"/></xsl:call-template>
          </td>
        </tr>
        
        <!-- description is mandatory so displayed even if empty -->
        <tr><td colspan="3" class="table_data_subsubheader">#requirement.Description#</td></tr>
        <tr>
          <td colspan="3" class="table_data_content">
            <xsl:call-template name="generateAttachments">
              <xsl:with-param name="parentType" select="'R_'"/>
              <xsl:with-param name="attachmentIcon" select="'resources/images/IconAttachment.gif'"/>
            </xsl:call-template>
            
            <xsl:value-of disable-output-escaping="yes" select="$requirementDescription" />
            &#160; <!-- to avoid having a ugly effect when there is no description -->
          </td>
        </tr>
        
        <!-- custom fields -->
        <xsl:call-template name="insertCustomFields">
          <xsl:with-param name="colSpan" select="'3'"/>
        </xsl:call-template>        
      </xsl:if>
      
      <!-- REQUIREMENT_TYPE_GENERIC -->
      <!-- REQUIREMENT_TYPE_JIRA -->
      <xsl:if test="$requirementTrackingType = 2 or $requirementTrackingType = 6">
        <tr>
          <td width="100%" class="table_data_subheader">#requirement.Link#</td>
        </tr>
        <tr>
          <td class="table_data_content">
            <!-- anchor link -->
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="$requirementUrl" />
              </xsl:attribute>
              <xsl:value-of select="$requirementUrl" />
            </a>
          </td>
        </tr>
      </xsl:if>
    </table>
    <br/>
  </xsl:template>

<!-- ========================= SPECIFICATIONS ============================== -->

  <xsl:template name="specificationIsFormalToString">
    <xsl:param name="isFormal"/>
    <xsl:if test="$isFormal='true'"><img src="resources/images/IconCheck.gif"/></xsl:if>
  </xsl:template>

  <xsl:template name="specificationPriorityToString">
    <xsl:param name="priority"/>
    <xsl:if test="$priority='10'"><span class="text_red">#specification.priority.High#</span></xsl:if>
    <xsl:if test="$priority='20'"><span class="text_blue">#specification.priority.Normal#</span></xsl:if>
    <xsl:if test="$priority='30'"><span class="text_grey">#specification.priority.Low#</span></xsl:if>
  </xsl:template>

  <xsl:template name="specificationStatusToString">
    <xsl:param name="status"/>
    <xsl:if test="$status='10'"><span class="text_red">#specification.status.New#</span></xsl:if>
    <xsl:if test="$status='20'"><span class="text_blue">#specification.status.Acknowledged#</span></xsl:if>
    <xsl:if test="$status='30'"><span class="text_green">#specification.status.Approved#</span></xsl:if>
  </xsl:template>


  <xsl:template name="generateUniqueSpecificationsDetailsFromAllLevels">
    <!-- we use this to search for all distinct specifications in the document -->
    <xsl:key name="distinct-specifications" match="specification" use="@id"/>
    <xsl:variable name="nodeSet" select="//specification[generate-id()=generate-id(key('distinct-specifications',@id))]" />

    <xsl:call-template name="generateSpecificationsDetails">
      <xsl:with-param name="nodeSet" select="$nodeSet"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateSpecificationsDetailsFromCurrentLevel">
    <xsl:call-template name="generateSpecificationsDetails">
      <xsl:with-param name="nodeSet" select="specification"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateSpecificationsDetails">
    <xsl:param name="nodeSet"/>

    <xsl:for-each select="$nodeSet">
      <xsl:call-template name="generateSpecificationDetails"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="generateSpecificationDetails">
    <xsl:variable name="specificationId"            select="@id" />
    <xsl:variable name="specificationName"          select="@name" />
    <xsl:variable name="specificationDescription"   select="description" />
    <xsl:variable name="specificationIsFormal"      select="@isFormal" />
    <xsl:variable name="specificationPriority"      select="@priority" />
    <xsl:variable name="specificationStatus"        select="@status" />

    <br/>
    <a>
      <xsl:attribute name="name">
        <xsl:call-template name="generateUniqueId">
          <xsl:with-param name="type" select="'specification'"/>
          <xsl:with-param name="id" select="@id"/>
        </xsl:call-template>
      </xsl:attribute>
    </a>

    <table width="100%">
      <tr>
        <td colspan="3" class="table_data_subheader">
          <xsl:call-template name="urlToObject">
            <xsl:with-param name="globalId" select="concat('S_', $specificationId)"/>
            <xsl:with-param name="name"     select="$specificationName"/> 
          </xsl:call-template>
        </td>
      </tr>
      <tr>
        <td width="33%" class="table_data_subsubheader">#specification.IsFormal#</td>
        <td width="33%" class="table_data_subsubheader">#specification.Priority#</td>
        <td width="33%" class="table_data_subsubheader">#specification.Status#</td>
      </tr>
      <tr>
        <td class="table_data_content">
          <xsl:call-template name="specificationIsFormalToString"><xsl:with-param name="isFormal" select="$specificationIsFormal"/></xsl:call-template>
        </td>
        <td class="table_data_content">
          <xsl:call-template name="specificationPriorityToString"><xsl:with-param name="priority" select="$specificationPriority"/></xsl:call-template>
        </td>
        <td class="table_data_content">
          <xsl:call-template name="specificationStatusToString"><xsl:with-param name="status" select="$specificationStatus"/></xsl:call-template>
        </td>
      </tr>
      
      <!-- description is mandatory so displayed even if empty -->
      <tr><td colspan="3" class="table_data_subsubheader">#specification.Description#</td></tr>
      <tr>
        <td colspan="3" class="table_data_content">
          <xsl:call-template name="generateAttachments">
            <xsl:with-param name="parentType" select="'S_'"/>
            <xsl:with-param name="attachmentIcon" select="'resources/images/IconAttachment.gif'"/>
          </xsl:call-template>
          
          <xsl:value-of disable-output-escaping="yes" select="$specificationDescription" />
          &#160; <!-- to avoid having a ugly effect when there is no description -->
        </td>
      </tr>
      
      <!-- custom fields -->
      <xsl:call-template name="insertCustomFields">
        <xsl:with-param name="colSpan" select="'3'"/>
      </xsl:call-template>

    </table>
    <br/>
  </xsl:template>


<!-- ============================= TESTS ================================== -->

  <xsl:template name="generateUniqueTestsDetailsFromAllLevels">
    <!-- we use this to search for all distinct tests in the document -->
    <xsl:key name="distinct-tests" match="test" use="@id"/>
    <xsl:variable name="nodeSet" select="//test[generate-id()=generate-id(key('distinct-tests',@id))]" />

    <xsl:call-template name="generateTestsDetails">
      <xsl:with-param name="nodeSet" select="$nodeSet"/>
    </xsl:call-template>
  </xsl:template>







  <xsl:template name="generateTestsDetailsFromCurrentLevel">
    <xsl:call-template name="generateTestsDetails">
      <xsl:with-param name="nodeSet" select="test"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="generateTestsDetails">
    <xsl:param name="nodeSet"/>

    <xsl:for-each select="$nodeSet">
      <xsl:call-template name="generateTestDetails"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="generateTestDetails">
    <xsl:variable name="testName"          select="@name" />
    <xsl:variable name="testId"            select="@id" />
    <xsl:variable name="author"            select="@author" />
    <xsl:variable name="developer"         select="@developer" />
    <xsl:variable name="testDescription"   select="description" />
    <xsl:variable name="testPrerequisites" select="prerequisites" />
    <xsl:variable name="nbTestcases"       select="count(testcase)" />

    <br/>
    <a>
      <xsl:attribute name="name">
        <xsl:call-template name="generateUniqueId">
          <xsl:with-param name="type" select="'test'"/>
          <xsl:with-param name="id" select="@id"/>
        </xsl:call-template>
      </xsl:attribute>
    </a>

    <table width="100%">
      <tr>
        <td class="table_data_subheader_bottom_align">
          <h4 class="no_margin">
            <img src="resources/images/IconTest.gif"/>&#160;
            <xsl:call-template name="urlToObject">
              <xsl:with-param name="globalId" select="concat('T_', $testId)"/>
              <xsl:with-param name="name"     select="$testName"/> 
            </xsl:call-template>            
          </h4>
        </td>
        <td class="table_data_subheader_bottom_align">
          <b>#testplan.Author#: </b> <xsl:value-of disable-output-escaping="yes" select="$author" />
          &#160;&#160;&#160;&#160;&#160;&#160;
          <b>#testplan.Developer#: </b> <xsl:value-of disable-output-escaping="yes" select="$developer" />
        </td>
      </tr>
      
      
      <!-- description is mandatory so displayed even if empty -->
      <tr><td colspan="2" class="table_data_subsubsubheader">#testplan.Description#</td></tr>
      <tr>
      	<td colspan="2" class="table_data_content">
           <xsl:call-template name="generateAttachments">
              <xsl:with-param name="parentType" select="'T_'"/>
              <xsl:with-param name="attachmentIcon" select="'resources/images/IconAttachment.gif'"/>
           </xsl:call-template>
      	   <xsl:value-of disable-output-escaping="yes" select="$testDescription" />
      	   &#160; <!-- to avoid having a ugly effect when there is no description -->
      	</td>
      </tr>
      
      <xsl:if test="string-length($testPrerequisites)>0">
        <tr><td colspan="2" class="table_data_subsubsubheader">#testplan.Prerequisites#</td></tr>
        <tr>
          <td colspan="2" class="table_data_content">
            <xsl:value-of disable-output-escaping="yes" select="$testPrerequisites" />
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="count(requirement)>0">
        <tr><td colspan="2" class="table_data_subsubsubheader">#testplan.Requirements#</td></tr>
        <xsl:for-each select="requirement">
          <xsl:variable name="reqName"          select="@name" />
          <xsl:variable name="reqId"            select="@id" />
          <xsl:variable name="reqDescription"   select="description" />
          <tr>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$reqId" />&#160; &#160; &#160;<xsl:value-of disable-output-escaping="yes" select="$reqName" />
            </td>
            <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$reqDescription" /></td>
          </tr>
        </xsl:for-each>
      </xsl:if>


      <xsl:if test="count(specification)>0">
        <tr><td colspan="2" class="table_data_subsubsubheader">#testplan.Specifications#</td></tr>
        <xsl:for-each select="specification">
          <xsl:variable name="specName"          select="@name" />
          <xsl:variable name="specId"            select="@id" />
          <xsl:variable name="specDescription"   select="description" />
          <tr>
            <td class="table_data_content">
              <xsl:value-of disable-output-escaping="yes" select="$specId" />&#160; &#160; &#160;<xsl:value-of disable-output-escaping="yes" select="$specName" />
            </td>
            <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$specDescription" /></td>
          </tr>
        </xsl:for-each>
      </xsl:if>
      
    </table>
    <br/>

    <xsl:if test="$nbTestcases>0">
      <xsl:for-each select="testcase">
        <xsl:sort select="@index" data-type="number"/>

        <xsl:variable name="testcaseId"                select="@id" />
        <xsl:variable name="testcaseName"              select="@name" />
        <xsl:variable name="testcaseIndex"             select="@index" />
        <xsl:variable name="testcaseImplemented"       select="@devCompleted" />
        <xsl:variable name="testcaseUseDescAsTestplan" select="@useDescAsTestplan" />
        <xsl:variable name="testcaseDescription"       select="description" />
        <xsl:variable name="nbStep"                    select="count(steps/object[@type='root']/object[@type='step'])" />
        <xsl:variable name="nbAttachment"              select="count(attachment)" />

        <!--
        table:        border missing
        margin:       overwrite the right border in Word
        padding:      no effect on Word
        margin in %:  no effect on Word
        margin-right: no effect on Word
        -->
        <!-- <div style="margin-left:50px"> -->
        <table border="0" width="100%"><tr><td width="10%">&#160;&#160;</td><td class="table_container" width="90%">
        
        <table border="0" width="100%">
          <tr>
            <td class="table_data_subsubheader_bottom_align" colspan="3">

              <h5 class="no_margin">
                <img src="resources/images/IconTestcase.gif"/>
                <xsl:if test="$testcaseImplemented='1'">
                  <img src="resources/images/IconImplemented.gif"/>
                </xsl:if>
                <xsl:if test="$testcaseImplemented='0'">
                  <img src="resources/images/IconNotImplemented.gif"/>
                </xsl:if>
                <xsl:call-template name="urlToObject">
                  <xsl:with-param name="globalId" select="concat('TC_', $testcaseId)"/>
                  <xsl:with-param name="name"     select="concat('(', $testcaseIndex, ')&#160;&#160;&#160;', $testcaseName)"/> 
                </xsl:call-template>
              </h5>
            </td>
          </tr>
          
          <xsl:if test="string-length($testcaseDescription)>0 or $nbAttachment>0">
            <tr>
              <td class="table_data_subsubsubheader" colspan="3">#testplan.Description#</td>
            </tr>
            <tr>
              <td  class="table_data_content2" colspan="3">
                <xsl:call-template name="generateAttachments">
                    <xsl:with-param name="parentType" select="'TC_'"/>
                    <xsl:with-param name="attachmentIcon" select="'resources/images/IconAttachment.gif'"/>
                </xsl:call-template>
                <xsl:value-of disable-output-escaping="yes" select="$testcaseDescription"/>
              </td>
            </tr>
          </xsl:if>
          
          <!-- custom fields -->
          <xsl:call-template name="insertCustomFields">
            <xsl:with-param name="colSpan" select="'3'"/>
          </xsl:call-template>

          <!-- steps -->
          <xsl:if test="$nbStep > 0">
            <tr>
              <td class="table_data_subsubsubheader">#testplan.Step#</td>
              <td class="table_data_subsubsubheader">#testplan.Parameters#</td>
              <td class="table_data_subsubsubheader">#testplan.ExpectedResult#</td>
            </tr>
            <xsl:for-each select="steps/object[@type='root']/object[@type='step']">
              <xsl:variable name="nbParameters" select="count(object[@type='parameters']/object[@type='parameter'])" />

              <tr>
                <!-- all the other lines: we add the description, parameters and checks -->
                <td class="table_data_content"><xsl:value-of select="@description" /></td>
                <td class="table_data_content">
                  <xsl:if test="$nbParameters = 0">
                    &#160;
                  </xsl:if>
                  <xsl:for-each select="object[@type='parameters']/object[@type='parameter']">
                    - <xsl:value-of select="@description" /><br/>
                  </xsl:for-each>
                </td>
                <td class="table_data_content">
                  <table>
                    <xsl:for-each select="object[@type='checks']/object[@type='operator']">
                      <xsl:call-template name="generateChecksDetails">
                        <xsl:with-param name="tab" select="''"/>
                      </xsl:call-template>
                    </xsl:for-each>
                  </table>
                </td>
              </tr>
            </xsl:for-each>
          </xsl:if>
        </table>
        </td></tr></table>
        <br/>
      </xsl:for-each>
      
    </xsl:if>
  </xsl:template>



  <!-- generate check detail -->
  <xsl:template name="generateChecksDetails">
    <xsl:param name="tab"/>

    <xsl:variable name="typeNode" select="@type" />
    <xsl:if test="$typeNode = 'operator'">
          <xsl:variable name="operatorValue" select="@operator" />
      <tr>
        <td width="100%" class="table_data_content2_no_border">
          <xsl:value-of disable-output-escaping="yes" select="$tab" />
          <xsl:if test="$operatorValue = 'and'">
            <img src="resources/images/IconAnd.gif"/>
          </xsl:if>

          <xsl:if test="$operatorValue = 'or'">
            <img src="resources/images/IconOr.gif"/>
          </xsl:if>

          <xsl:if test="$operatorValue = 'not'">
            <img src="resources/images/IconNot.gif"/>
          </xsl:if>
        </td>
      </tr>
    </xsl:if>
    <xsl:if test="$typeNode = 'check'">
      <tr>
        <td width="100%" class="table_data_content2_no_border">
          <xsl:value-of disable-output-escaping="yes" select="$tab" />
          <img src="resources/images/IconItem.gif"/>&#160;<xsl:value-of select="@description" />
        </td>
      </tr>
    </xsl:if>

    <xsl:for-each select="object[@type='operator']|object[@type='check']">
      <xsl:call-template name="generateChecksDetails">
        <xsl:with-param name="tab" select="concat($tab, '&#160;&#160;&#160;&#160;')"/>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>


<!-- ========================= CAMPAIGN SESSION ============================ -->

  <xsl:template name="generateShortCampaignSession">
      <xsl:variable name="campaignSessionId"         select="@id" />
      <xsl:variable name="campaignSessionName"       select="@name" />
      <xsl:variable name="campaignSessionStartDate"  select="@start_date" />
      <xsl:variable name="campaignSessionStopDate"   select="@stop_date" />

      <tr>
        <td class="table_data_content">
            <!-- anchor link -->
            <xsl:variable name="campaignSessionSharp" select="concat('#CS_', $campaignSessionId)" />
            <a>
              <xsl:attribute name="href">
                <!--<xsl:value-of select="$campaignSessionSharp" />-->
                <xsl:call-template name="generateSharpUniqueId">
                  <xsl:with-param name="type" select="'campaignSession'"/>
                  <xsl:with-param name="id" select="@id"/>
                </xsl:call-template>
              </xsl:attribute>
              [CS_<xsl:value-of select="$campaignSessionId"/>]&#160;<xsl:value-of select="$campaignSessionName" />
            </a>
        </td>
        <td class="table_data_content">
          <xsl:value-of select="$campaignSessionStartDate"/>
        </td>
        <td class="table_data_content">
          <xsl:value-of select="$campaignSessionStopDate"/>
        </td>
      </tr>
  </xsl:template>





<!-- ============================== UTILITIES ============================== -->


  <xsl:template name="insertCustomFields">
    <xsl:param name="colSpan" />
  
    <xsl:for-each select="customField">
      <xsl:sort select="@position" data-type="number"/>
  
      <xsl:variable name="customFieldType" select="@type" />
      <xsl:variable name="customFieldName" select="@name" />
      <xsl:variable name="customFieldMandatory" select="@mandatory" />
      <xsl:variable name="customFieldValue" select="@value" />
      <xsl:if test="$customFieldType='html'">
        <xsl:variable name="customFieldValue" select="." />
      </xsl:if>
      
      <tr>
        <td class="table_data_subsubheader">
          <xsl:attribute name="colspan">
            <xsl:value-of select="$colSpan" />
          </xsl:attribute>
          <xsl:value-of select="$customFieldName" />
        </td>
      </tr>
      <tr>
        <td class="table_data_content">
          <xsl:attribute name="colspan">
            <xsl:value-of select="$colSpan" />
          </xsl:attribute>
          <xsl:if test="$customFieldType='html'">
            <xsl:value-of disable-output-escaping="yes" select="." />
          </xsl:if>
          <xsl:if test="$customFieldType!='html'">
            <xsl:value-of select="@value" />
          </xsl:if>
        </td>
      </tr>
    </xsl:for-each>     
  </xsl:template>

  <xsl:template name="generateReferences">
    <xsl:param name="startTag" />
    <xsl:param name="stopTag" />
    <xsl:param name="string" />
    <xsl:param name="armed"/>
    <xsl:param name="type"/>
    
    <xsl:choose>
      <xsl:when test="$armed and contains($string, $stopTag)">
      
        <xsl:if test="$type='TBD'">
          <span class="TBD"><xsl:value-of select="substring-before($string, $stopTag)"/></span><br/>
        </xsl:if>
        <xsl:if test="$type='ABBR'">
          <span class="ABBR"><xsl:value-of select="substring-before($string, $stopTag)"/></span><br/>
        </xsl:if>
        
        <xsl:call-template name="generateReferences">
          <xsl:with-param name="startTag" select="$startTag"/>
          <xsl:with-param name="stopTag"  select="$stopTag"/>
          <xsl:with-param name="string"   select="substring-after($string, $stopTag)" />
          <xsl:with-param name="armed"    select="0" />
          <xsl:with-param name="type"     select="$type" />
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="not($armed) and contains($string, $startTag)">
        <xsl:call-template name="generateReferences">
          <xsl:with-param name="startTag" select="$startTag"/>
          <xsl:with-param name="stopTag"  select="$stopTag"/>
          <xsl:with-param name="string"   select="substring-after($string, $startTag)" />
          <xsl:with-param name="armed"    select="1" />
          <xsl:with-param name="type"     select="$type" />
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="generateTBD">
      <xsl:variable name="tBDParentNodes" select="(//campaign | //*[count(descendant::test)>0 or count(descendant::testcase)>0])
                                                   /*[description[contains(., '&lt;span class=&quot;TBD&quot;&gt;') and contains(., '&lt;/span&gt;')] | 
                                                      prerequisites[contains(., '&lt;span class=&quot;TBD&quot;&gt;') and contains(., '&lt;/span&gt;')]]" />
      <xsl:if test="count($tBDParentNodes)>0">
        <a name="_tbd"/>
        <h1>To Be Defined</h1>
          <xsl:for-each select="$tBDParentNodes">
            <xsl:for-each select="description | prerequisites">
              <xsl:call-template name="generateReferences">
                <xsl:with-param name="startTag" select="'&lt;span class=&quot;TBD&quot;&gt;'"/>
                <xsl:with-param name="stopTag"  select="'&lt;/span&gt;'"/>
                <xsl:with-param name="string"   select="."/>
                <xsl:with-param name="armed"    select="0"/>
                <xsl:with-param name="type"     select="'TBD'"/>
              </xsl:call-template>
            </xsl:for-each>
          </xsl:for-each>
      </xsl:if>
  </xsl:template>
          
  <xsl:template name="generateABBR">
      <xsl:variable name="aBBRParentNodes" select="(//campaign | //*[count(descendant::test)>0 or count(descendant::testcase)>0])
                                                    /*[description[contains(., '&lt;span class=&quot;ABBR&quot;&gt;') and contains(., '&lt;/span&gt;')] | 
                                                       prerequisites[contains(., '&lt;span class=&quot;ABBR&quot;&gt;') and contains(., '&lt;/span&gt;')]]" />
      <xsl:if test="count($aBBRParentNodes)>0">
        <a name="_tbd"/>
        <h1>Abbreviations</h1>
          <xsl:for-each select="$aBBRParentNodes">
            <xsl:for-each select="description | prerequisites">
              <xsl:call-template name="generateReferences">
                <xsl:with-param name="startTag" select="'&lt;span class=&quot;ABBR&quot;&gt;'"/>
                <xsl:with-param name="stopTag"  select="'&lt;/span&gt;'"/>
                <xsl:with-param name="string"   select="."/>
                <xsl:with-param name="armed"    select="0"/>
                <xsl:with-param name="type"     select="'ABBR'"/>
              </xsl:call-template>
            </xsl:for-each>
          </xsl:for-each>
      </xsl:if>
  </xsl:template>

</xsl:stylesheet>