<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="xml" indent="yes" encoding="UTF-8"/>
	
	<xsl:template name="conf">
		<xsl:for-each select="formItem">
			<tr>
				<td width="320px" class="table_data_content"><xsl:value-of select="@name" /></td>
				<td class="table_data_content">
					<xsl:call-template name="confValue">
						<xsl:with-param name="type" select="@type"/>
						<xsl:with-param name="value" select="@value"/>
					</xsl:call-template>
				</td>
			</tr>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="confValue">
	    <xsl:param name="type"/>
	    <xsl:param name="value"/>

		<xsl:if test="$type='boolean'">
			<xsl:if test="$value='0'">false</xsl:if>
			<xsl:if test="$value='1'">true</xsl:if>
		</xsl:if>
		<xsl:if test="$type!='boolean'">
			<xsl:value-of select="$value" />
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="statusToString">
		<xsl:param name="status"/>
		<xsl:if test="$status='1'">#report.Success#</xsl:if>				
		<xsl:if test="$status='2'">#report.Failure#</xsl:if>				
		<xsl:if test="$status='0'">#report.Not_executed#</xsl:if>				
		<xsl:if test="$status='6'">#report.Unknown#</xsl:if>				
		<xsl:if test="$status='3'">#report.Relative#</xsl:if>
	</xsl:template>

	<xsl:template name="statusToIcon">
		<xsl:param name="status"/>
		<xsl:if test="$status='1'"><img src="resources/images/IconResultSuccess.gif"/></xsl:if>				
		<xsl:if test="$status='2'"><img src="resources/images/IconResultFailure.gif"/></xsl:if>				
		<xsl:if test="$status='0'"><img src="resources/images/IconResultNotExecuted.gif"/></xsl:if>				
		<xsl:if test="$status='6'"><img src="resources/images/IconResultUnknown.gif"/></xsl:if>				
		<xsl:if test="$status='3'"><img src="resources/images/IconResultRelative.gif"/></xsl:if>
	</xsl:template>

	<xsl:template name="statusToIconAndString">
		<xsl:param name="status"/>
    <table>
      <tr>
        <td class="table_message_content">
		      <xsl:call-template name="statusToIcon"><xsl:with-param name="status" select="$status"/></xsl:call-template>
        </td>
        <td class="table_message_content" width="65px">						
		      <xsl:call-template name="statusToString"><xsl:with-param name="status" select="$status"/></xsl:call-template>
        </td>
      </tr>
    </table>						
	</xsl:template>

	<xsl:template name="testAndTestcaseStats">
		<xsl:param name="graphInReportsEnabled"/>
		<table border="0" cellpading="0" cellspacing="0">
			<tr>
				<td valign="top"><xsl:call-template name="testStats"/></td><td>&#160;&#160;&#160;&#160;&#160;</td><td valign="top"><xsl:call-template name="testcaseStats"/></td>
			</tr>
			<xsl:choose>
        <xsl:when test="$graphInReportsEnabled='true'">			
		      <tr>
  		      <td valign="top"><br/><img src="resources/images/chart_results_testsSF.jpg"/></td><td></td>
  					<td valign="top"><br/><img src="resources/images/chart_results_testcasesSF.jpg"/></td>
					</tr>
				</xsl:when>
				<xsl:otherwise>
					<tr>
						<td valign="top"><xsl:call-template name="testGraph"/></td><td></td>
						<td valign="top"><xsl:call-template name="testcaseGraph"/></td>
					</tr>
				</xsl:otherwise>
	    </xsl:choose>
		</table>
		<br/>
		<xsl:if test="$graphInReportsEnabled='true'">	
  		SUT coverage:
  		<table border="0" cellpading="0" cellspacing="0">
        <tr>
					<td valign="top"><br/><img src="resources/images/chart_results_tests.jpg"/></td>
					<td>&#160;&#160;&#160;&#160;&#160;</td>
					<td valign="top"><br/><img src="resources/images/chart_results_testcases.jpg"/></td>
				</tr>
  		</table>
    </xsl:if>		
	</xsl:template>

	<xsl:template name="testStats">
		<xsl:variable name="nbSuccess" select="count(//test[@status='1'])" />
		<xsl:variable name="nbFailure" select="count(//test[@status='2'])" />
		<xsl:variable name="nbUnknown" select="count(//test[@status='6'])" />
		<xsl:variable name="nbOther"   select="count(//test) - $nbSuccess - $nbFailure - $nbUnknown" />
		<xsl:variable name="nbTotal"   select="count(//test)" />

		<xsl:variable name="percentSuccess" select="round(($nbSuccess div $nbTotal)*100)" />
		<xsl:variable name="percentFailure" select="round(($nbFailure div $nbTotal)*100)" />
		<xsl:variable name="percentUnknown" select="round(($nbUnknown div $nbTotal)*100)" />
		<xsl:variable name="percentOther"   select="round(($nbOther div $nbTotal)*100)" />

		<table border="0" width="220px">
			<tr>
				<td class="table_label" colspan="3" align="left"><u>#report.Tests#</u></td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_success#:</td>
				<td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbSuccess" /></td>
				<td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentSuccess" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_failure#:</td>
				<td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbFailure" /></td>
				<td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentFailure" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_unknown#:</td>
				<td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbUnknown" /></td>
				<td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentUnknown" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_tests#:</td>
				<td colspan="2" class="table_value" align="right"><i><xsl:value-of disable-output-escaping="yes" select="$nbTotal" /></i></td>
			</tr>
		</table>
	</xsl:template>

	<xsl:template name="testcaseStats">
		<xsl:variable name="nbSuccess"     select="count(//test/testcase[@status='1'])" />
		<xsl:variable name="nbFailure"     select="count(//test/testcase[@status='2'])" />
		<xsl:variable name="nbNotExecuted" select="count(//test/testcase[@status='0'])" />
		<xsl:variable name="nbRelative"    select="count(//test/testcase[@status='3'])" />
		<xsl:variable name="nbOther"       select="count(//test/testcase) - $nbSuccess - $nbFailure - $nbNotExecuted - $nbRelative" />
		<xsl:variable name="nbTotal"       select="count(//test/testcase)" />

		<xsl:variable name="percentSuccess"     select="round(($nbSuccess div $nbTotal)*100)" />
		<xsl:variable name="percentFailure"     select="round(($nbFailure div $nbTotal)*100)" />
		<xsl:variable name="percentNotExecuted" select="round(($nbNotExecuted div $nbTotal)*100)" />
		<xsl:variable name="percentRelative"    select="round(($nbRelative div $nbTotal)*100)" />
		<xsl:variable name="percentOther"       select="round(($nbOther div $nbTotal)*100)" />

		<table border="0" width="220px">
			<tr>
				<td class="table_label" colspan="3" align="left"><u>#report.Testcases#</u></td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_success#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbSuccess" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentSuccess" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_failure#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbFailure" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentFailure" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_not_executed#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbNotExecuted" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentNotExecuted" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_relative#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbRelative" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentRelative" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_testcases#:</td>
			    <td colspan="2" class="table_value" align="right"><i><xsl:value-of disable-output-escaping="yes" select="$nbTotal" /></i></td>
			</tr>
		</table>
	</xsl:template>

	<xsl:template name="testGraph">
		<xsl:variable name="nbSuccess" select="count(//test[@status='1'])" />
		<xsl:variable name="nbFailure" select="count(//test[@status='2'])" />
		<xsl:variable name="nbUnknown" select="count(//test[@status='6'])" />
		<xsl:variable name="nbOther"   select="count(//test) - $nbSuccess - $nbFailure - $nbUnknown" />
		<xsl:variable name="nbTotal"   select="count(//test)" />

		<xsl:variable name="percentSuccess" select="round(($nbSuccess div $nbTotal)*100)" />
		<xsl:variable name="percentFailure" select="round(($nbFailure div $nbTotal)*100)" />
		<xsl:variable name="percentUnknown" select="round(($nbUnknown div $nbTotal)*100)" />
		<xsl:variable name="percentOther"   select="round(($nbOther div $nbTotal)*100)" />

		<table border="0" cellpading="0" cellspacing="0" width="100%">
			<tr>
				<td height="10px" class="bg_success"><xsl:attribute name="width"><xsl:value-of select="$percentSuccess" />%</xsl:attribute></td>
				<td height="10px" class="bg_failure"><xsl:attribute name="width"><xsl:value-of select="$percentFailure" />%</xsl:attribute></td>
				<td height="10px" class="bg_unknown"><xsl:attribute name="width"><xsl:value-of select="$percentUnknown" />%</xsl:attribute></td>
				<td height="10px" class="bg_other"><xsl:attribute name="width"><xsl:value-of select="$percentOther" />%</xsl:attribute></td>
			</tr>
		</table>
	</xsl:template>

	<xsl:template name="testcaseGraph">
		<xsl:variable name="nbSuccess"     select="count(//test/testcase[@status='1'])" />
		<xsl:variable name="nbFailure"     select="count(//test/testcase[@status='2'])" />
		<xsl:variable name="nbNotExecuted" select="count(//test/testcase[@status='0'])" />
		<xsl:variable name="nbRelative"    select="count(//test/testcase[@status='3'])" />
		<xsl:variable name="nbOther"       select="count(//test/testcase) - $nbSuccess - $nbFailure - $nbNotExecuted - $nbRelative" />
		<xsl:variable name="nbTotal"       select="count(//test/testcase)" />

		<xsl:variable name="percentSuccess"     select="round(($nbSuccess div $nbTotal)*100)" />
		<xsl:variable name="percentFailure"     select="round(($nbFailure div $nbTotal)*100)" />
		<xsl:variable name="percentNotExecuted" select="round(($nbNotExecuted div $nbTotal)*100)" />
		<xsl:variable name="percentRelative"    select="round(($nbRelative div $nbTotal)*100)" />
		<xsl:variable name="percentOther"       select="round(($nbOther div $nbTotal)*100)" />
	
		<table border="0" cellpading="0" cellspacing="0" width="100%">
			<tr>
				<td height="10px" class="bg_success"><xsl:attribute name="width"><xsl:value-of select="$percentSuccess" />%</xsl:attribute></td>
				<td height="10px" class="bg_failure"><xsl:attribute name="width"><xsl:value-of select="$percentFailure" />%</xsl:attribute></td>
				<td height="10px" class="bg_not_executed"><xsl:attribute name="width"><xsl:value-of select="$percentNotExecuted" />%</xsl:attribute></td>
				<td height="10px" class="bg_relative"><xsl:attribute name="width"><xsl:value-of select="$percentRelative" />%</xsl:attribute></td>
				<td height="10px" class="bg_other"><xsl:attribute name="width"><xsl:value-of select="$percentOther" />%</xsl:attribute></td>
			</tr>
		</table>
	</xsl:template>
	
	<xsl:template name="testcaseDetailsStats">
		<xsl:variable name="nbSuccess"     select="count(testcase[@status='1'])" />
		<xsl:variable name="nbFailure"     select="count(testcase[@status='2'])" />
		<xsl:variable name="nbNotExecuted" select="count(testcase[@status='0'])" />
		<xsl:variable name="nbRelative"    select="count(testcase[@status='3'])" />
		<xsl:variable name="nbOther"       select="count(testcase) - $nbSuccess - $nbFailure - $nbNotExecuted - $nbRelative" />
		<xsl:variable name="nbTotal"       select="count(testcase)" />

		<xsl:variable name="percentSuccess"     select="round(($nbSuccess div $nbTotal)*100)" />
		<xsl:variable name="percentFailure"     select="round(($nbFailure div $nbTotal)*100)" />
		<xsl:variable name="percentNotExecuted" select="round(($nbNotExecuted div $nbTotal)*100)" />
		<xsl:variable name="percentRelative"    select="round(($nbRelative div $nbTotal)*100)" />
		<xsl:variable name="percentOther"       select="round(($nbOther div $nbTotal)*100)" />

		<table border="0" width="220px">
			<tr>
				<td colspan="3" class="table_label" align="left"><u>#report.Testcases#</u></td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_success#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbSuccess" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentSuccess" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_failure#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbFailure" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentFailure" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_not_executed#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbNotExecuted" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentNotExecuted" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_relative#:</td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbRelative" /></td>
			    <td class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$percentRelative" />%</td>
			</tr>
			<tr>
				<td class="table_label" align="left">#report.Nb_testcases#:</td>
			    <td colspan="2" class="table_value" align="right"><xsl:value-of disable-output-escaping="yes" select="$nbTotal" /></td>
			</tr>
		</table>
	</xsl:template>

	<!--
	<xsl:template name="testcaseDetailsGraph">
		<xsl:variable name="nbSuccess"     select="count(testcase[@status='1'])" />
		<xsl:variable name="nbFailure"     select="count(testcase[@status='2'])" />
		<xsl:variable name="nbNotExecuted" select="count(testcase[@status='0'])" />
		<xsl:variable name="nbRelative"    select="count(testcase[@status='3'])" />
		<xsl:variable name="nbOther"       select="count(testcase) - $nbSuccess - $nbFailure - $nbNotExecuted - $nbRelative" />
		<xsl:variable name="nbTotal"       select="count(testcase)" />

		<xsl:variable name="percentSuccess"     select="round(($nbSuccess div $nbTotal)*100)" />
		<xsl:variable name="percentFailure"     select="round(($nbFailure div $nbTotal)*100)" />
		<xsl:variable name="percentNotExecuted" select="round(($nbNotExecuted div $nbTotal)*100)" />
		<xsl:variable name="percentRelative"    select="round(($nbRelative div $nbTotal)*100)" />
		<xsl:variable name="percentOther"       select="round(($nbOther div $nbTotal)*100)" />
	
		<table border="0" cellpading="0" cellspacing="0" width="100%">
			<tr>
				<td height="10px" class="bg_success"><xsl:attribute name="width"><xsl:value-of select="$percentSuccess" />%</xsl:attribute></td>
				<td height="10px" class="bg_failure"><xsl:attribute name="width"><xsl:value-of select="$percentFailure" />%</xsl:attribute></td>
				<td height="10px" class="bg_not_executed"><xsl:attribute name="width"><xsl:value-of select="$percentNotExecuted" />%</xsl:attribute></td>
				<td height="10px" class="bg_relative"><xsl:attribute name="width"><xsl:value-of select="$percentRelative" />%</xsl:attribute></td>
				<td height="10px" class="bg_other"><xsl:attribute name="width"><xsl:value-of select="$percentOther" />%</xsl:attribute></td>
			</tr>
		</table>
	</xsl:template>
	-->

	<xsl:template name="testcaseMessage">
	   <table>
	   <xsl:for-each select="message/step">
	      <xsl:variable name="stepTimeStamp" select="@timeStamp" />
	      <xsl:variable name="stepStatus"    select="@status" />
	      <xsl:variable name="stepMessage"   select="@message" />

	      <tr>								      
	         <td class="table_message_content"><xsl:value-of disable-output-escaping="yes" select="$stepTimeStamp" /></td> 
	      
	         <td class="table_message_content">
	            <xsl:call-template name="statusToIconAndString">
			           <xsl:with-param name="status" select="$stepStatus"/>
		          </xsl:call-template>
		       </td>
		     
		       <!--<td class="table_message_content">&#160;&#160;</td>-->
	         <td class="table_message_content"><xsl:value-of disable-output-escaping="yes" select="$stepMessage" /></td>
	      </tr>
	      
	   </xsl:for-each> <!-- message steps -->
	   </table>
	</xsl:template>


<xsl:template name="insertCharts">
    <xsl:param name="graphInReportsEnabled"/>
    <br/>
    <xsl:if test="$graphInReportsEnabled='true'">
      <table border="0" cellpading="0" cellspacing="0">
      
        <xsl:variable name="agentNodeSet" select="agent" /> <!-- store the agent nodeSet fore later reuse -->
        <xsl:for-each select="configurations/configuration">
          <xsl:variable name="categoryId"   select="@categoryId" />
          <xsl:variable name="categoryName" select="@categoryName" />
          <xsl:for-each select="$agentNodeSet">
            <xsl:variable name="agentId"   select="@id" />
            <xsl:variable name="agentName" select="@name" />
            <tr><td valign="top">#report.Category# <b><xsl:value-of disable-output-escaping="yes" select="$categoryName" /></b> - #report.Agent# <b><xsl:value-of disable-output-escaping="yes" select="$agentName" /></b>:</td></tr>
            <tr>
    					<td valign="top">
                <img alt="&lt;no data in graph&gt;">
                  <xsl:attribute name="src">
                    resources/images/<xsl:value-of select="$categoryId" />_<xsl:value-of select="$agentId" />_line_chart.jpg
                  </xsl:attribute>
                </img>
              </td>
    				</tr>
				  </xsl:for-each>
				</xsl:for-each>
				
			</table>  				
			<br/><br/>
		</xsl:if> 
    <br/>                  
        
    <table width="100%">
      <tr>
        <td valign="top">
          <xsl:call-template name="testAndTestcaseStats">
          	<xsl:with-param name="graphInReportsEnabled" select="$graphInReportsEnabled"/>
          </xsl:call-template>
        </td>
        <td width="100%" valigh="top">&#160;</td>
      </tr>
    </table>
    <br/>
</xsl:template>

</xsl:stylesheet>
