<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html" indent="yes" encoding="UTF-8"/>

  <!-- global variables -->
  <xsl:variable name="codebase" select="testplans/@codebase" />
  <xsl:variable name="bugtrackingType" select="testplans/@bugTrackingType" />
  <xsl:variable name="requirementTrackingType" select="testplans/@requirementTrackingType" />
  <xsl:variable name="specificationInLifecycle" select="testplans/@specificationInLifecycle" />

  <xsl:include href="#export_absolute_path#xsl/lib.xslt"/>
  <xsl:include href="#export_absolute_path#xsl/testplanHTML_Lib.xslt"/>
  <xsl:template match="/">

    <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">

	  <head>		
		<title>XStudio Report</title>
		<meta name="description" content="XStudio"/>
		<meta name="keywords" content="xstudio, report"/>
		<meta name="author" lang="en" content="XQual"/>
		<meta name="robots" content="noarchive"/>
		<meta http-equiv="Pragma" content="no-cache"/>
	  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
		<meta http-equiv="Content-Language" content="en"/>		
		<link rel="stylesheet" href="#css#" type="text/css"/>		
	  </head>

      <body>
        <xsl:variable name="nbCategory"            select="count(testplans/category)" />
        <xsl:variable name="domain"                select="testplans/@domain" />
        <xsl:variable name="graphInReportsEnabled" select="testplans/@graphInReportsEnabled" />
        <xsl:variable name="documentDate"          select="testplans/@date" />
        <xsl:variable name="documentAuthor"        select="testplans/@author" />

        <div align="center">
          <table width="100%">
            <tr>
              <td valign="top"><img src="#images#/#company_logo#"/></td>
            </tr>
          </table>
          <br/>

          <xsl:if test="$domain = 1">
             <span class="report_title">#testplan.Complete_Testplan#</span><br/>
          </xsl:if>
          <xsl:if test="$domain = 2">
            <span class="report_title">#testplan.Category_Testplan#</span><br/>
          </xsl:if>
          <xsl:if test="$domain = 3">
            <span class="report_title">#testplan.Folder_Testplan#</span><br/>
          </xsl:if>
          <xsl:if test="$domain = 4">
            <span class="report_title">#testplan.Campaign_Testplan#</span><br/>
          </xsl:if>
          <span class="report_subtitle">Basic Without Statistics</span><br/>
          <span>Generated by <b><xsl:value-of select="$documentAuthor" /></b> (<b><xsl:value-of select="$documentDate" /></b>)</span><br/>
        </div>
        <br/><br/><br/><br/>
        
        <a name="_details"/>
        <h1>#testplan.Details#</h1>
        <xsl:call-template name="insertCharts">
          <xsl:with-param name="graphInReportsEnabled" select="$graphInReportsEnabled"/>
          <xsl:with-param name="domain" select="$domain"/>
        </xsl:call-template>


        <xsl:if test="$domain = 4"> <!-- campaign testplan -->
          <xsl:variable name="campaignName"        select="testplans/campaign/@name" />
          <xsl:variable name="campaignDescription" select="testplans/campaign/description" />

          <h2>
            <img src="#images#/IconCampaign.gif"/>&#160;
            Campaign: <xsl:value-of disable-output-escaping="yes" select="$campaignName" />&#160;
          </h2>
          <!-- folder description -->
          <xsl:if test="string-length($campaignDescription)>0">
            <p><xsl:value-of disable-output-escaping="yes" select="$campaignDescription" /></p>
          </xsl:if>
        </xsl:if>
        
        <!-- ******************* TESTPLAN DETAILS *************************** -->
        <xsl:if test="$nbCategory > 0">
          <xsl:for-each select="testplans/category[count(descendant-or-self::test)>0]">
            <xsl:variable name="categoryName" select="@name" />
            
            <!-- Categories -->
            <a>
              <xsl:attribute name="name">
                <xsl:call-template name="generateUniqueId">
                  <xsl:with-param name="type" select="'category'"/>
                  <xsl:with-param name="id" select="@id"/>
                </xsl:call-template>
              </xsl:attribute>
            </a>
            <h2>
              <img src="#images#/IconCategory.gif"/>&#160;
              Category: <xsl:value-of disable-output-escaping="yes" select="$categoryName" />&#160;
            </h2>

            <!-- Attachments -->
            <xsl:call-template name="generateAttachments">
              <xsl:with-param name="parentType"     select="'CT_'"/>
              <xsl:with-param name="attachmentIcon" select="'#images#/IconAttachment.gif'"/>
            </xsl:call-template>

            <!-- Tests -->
            <xsl:for-each select="folder[count(descendant::test) > 0]"> <!-- recursive call to display folders -->
                <xsl:call-template name="generateDetailsFolder">
                  <xsl:with-param name="parentName" select="$categoryName"/>
                </xsl:call-template>
            </xsl:for-each>

          </xsl:for-each>
        </xsl:if>
        <xsl:if test="$nbCategory = 0">
          <!-- Folders -->
          <xsl:for-each select="testplans/folder[count(descendant::test) > 0]"> <!-- recursive call to display folders -->
              <xsl:call-template name="generateDetailsFolder">
                <xsl:with-param name="parentName" select="''"/>
              </xsl:call-template>
          </xsl:for-each>
        </xsl:if>
        <br/><br/><br/><br/>
        
        <a name="_tbd"/>
        <xsl:call-template name="generateTBD"/>
        <br/><br/><br/>
       
        <a name="_abbr"/>
        <xsl:call-template name="generateABBR"/>
        <br/><br/><br/>
      </body>
    </html>

  </xsl:template>













  <xsl:template name="generateDetailsFolder">
    <xsl:param name="parentName"/>

    <xsl:variable name="folderName" select="@name" />
    <xsl:variable name="folderDescription" select="description" />

    <xsl:if test="count(test) > 0">
      
      <h3><xsl:value-of select="concat($parentName, '/', $folderName)" /></h3>
      <!-- folder description -->
      <p><xsl:value-of disable-output-escaping="yes" select="$folderDescription" /></p>

      <xsl:call-template name="generateAttachments">
        <xsl:with-param name="parentType" select="'FL_'"/>
        <xsl:with-param name="attachmentIcon" select="'#images#/IconAttachment.gif'"/>
      </xsl:call-template>

      <xsl:call-template name="generateTestsDetailsFromCurrentLevel"/>
    </xsl:if>

    <!-- Recursivity -->
    <xsl:for-each select="folder">
      <xsl:if test="count(descendant::test) > 0">
        <xsl:call-template name="generateDetailsFolder">
          <xsl:with-param name="parentName" select="concat($parentName, '/', $folderName)"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="generateFolderDetails">
    <xsl:variable name="nbTests"            select="count(descendant-or-self::test)"/>
    <xsl:variable name="nbTestcases"        select="count(descendant-or-self::testcase)"/>
    <xsl:variable name="nbTestcasesNotImpl" select="count(descendant-or-self::testcase[@devCompleted='0'])"/>
    <xsl:variable name="nbTestcasesImpl"    select="count(descendant-or-self::testcase[@devCompleted='1'])"/>
    <xsl:variable name="completion"         select="format-number(100 * $nbTestcasesImpl div $nbTestcases, '##.##')"/>

    <xsl:if test="$nbTests != 0">
      <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$nbTests"/></td>
    </xsl:if>
    <xsl:if test="$nbTests = 0">
      <td class="table_data_content"></td>
    </xsl:if>

    <xsl:if test="$nbTestcases != 0">
      <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="$nbTestcases"/></td>
      <xsl:if test="$nbTestcasesNotImpl != 0">
        <td class="table_data_content_red"><xsl:value-of disable-output-escaping="yes" select="$nbTestcasesNotImpl"/></td>
      </xsl:if>
      <xsl:if test="$nbTestcasesNotImpl = 0">
        <td class="table_data_content_red">&#160;</td>
      </xsl:if>
      <xsl:if test="$nbTestcasesImpl != 0">
        <td class="table_data_content_green"><xsl:value-of disable-output-escaping="yes" select="$nbTestcasesImpl"/></td>
      </xsl:if>
      <xsl:if test="$nbTestcasesImpl = 0">
        <td class="table_data_content_green">&#160;</td>
      </xsl:if>
      <xsl:if test="$completion != 0">
        <td class="table_data_content"><xsl:value-of disable-output-escaping="yes" select="concat($completion, ' %')"/></td>
      </xsl:if>
      <xsl:if test="$completion = 0">
        <td class="table_data_content">&#160;</td>
      </xsl:if>
    </xsl:if>
    <xsl:if test="$nbTestcases = 0">
      <td class="table_data_content">&#160;</td>
      <td class="table_data_content">&#160;</td>
      <td class="table_data_content">&#160;</td>
      <td class="table_data_content">&#160;</td>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>